#!/usr/bin/env python
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

import firenado.conf
from firenado.tornadoweb import get_request, TornadoApplication, TornadoHandler
from firenado.launcher import FirenadoLauncher, TornadoLauncher
from firenado.tornadoweb import TornadoComponent
import unittest
from tests import chdir_app
import os


class MainHandler(TornadoHandler):
    """ Basic handler rendering an index.html template
    """

    def get(self):
        self.render("index.html", initial_message="Hello world!!!")


class TestComponent(TornadoComponent):
    """ Enabled component referenced at the application configuration file
    """

    def get_handlers(self):
        return [
            (r'/', MainHandler),
        ]


class FakeRequest:

    def __init__(self):
        self.connection = FakeConnection()


class FakeConnection:

    def set_close_callback(self, callback):
        pass


class DisabledTestComponent(TornadoComponent):
    """ Disabled component referenced at the application configuration file
    """

    def get_handlers(self):
        return [
            (r'/', MainHandler),
        ]


class ApplicationComponentTestCase(unittest.TestCase):
    """ Case that tests an Firenado application after being loaded from its
    configuration file.
    """

    def setUp(self):
        """ Application configuration file will be read and components will be
        loaded.
        """
        chdir_app('tornadoweb')
        self.application = TornadoApplication()

    def test_component_loaded(self):
        """ Checks if test component was loaded correctly by the application
        __init__ method.
        """
        import tests.tornadoweb_test
        self.assertTrue('test' in self.application.components)
        self.assertTrue(isinstance(self.application.components['test'],
                                   tests.tornadoweb_test.TestComponent))
        self.assertFalse('disabled' in self.application.components)

    def test_static_path(self):
        """ Checks the static_path was placed in the application settings.
        """
        static_path_x = self.application.settings['static_path'].split("/")
        self.assertEqual(firenado.conf.app['static_path'], static_path_x[-1])


class GetRequestTestCase(unittest.TestCase):

    def test_get_request_simple(self):
        expected = "http://test"
        request = get_request(expected)
        self.assertEqual(expected, request.url)
        self.assertEqual(0, len(request.headers))
        self.assertEqual("GET", request.method)

    def test_get_request_with_path(self):
        expected = "http://test/a_path/informed"
        base_url = "http://test"
        request = get_request(url=base_url, path="a_path/informed")
        self.assertEqual(expected, request.url)
        base_url = "http://test/"
        request = get_request(url=base_url, path="a_path/informed")
        self.assertEqual(expected, request.url)

    def test_get_request_with_method(self):
        method = "POST"
        request = get_request(url="http://test", method=method)
        self.assertEqual(method, request.method)

    def test_get_request_with_form_urlencoded(self):
        request = get_request(url="http://test", form_urlencoded=True)
        self.assertEqual(1, len(request.headers))
        self.assertEqual("application/x-www-form-urlencoded",
                         request.headers['Content-Type'])


class TornadoHandlerTestCase(unittest.TestCase):
    """ TornadoHandler tests
    """

    def setUp(self):
        """ Application configuration file will be read and components will be
        loaded.
        """
        chdir_app('tornadoweb')
        self.application = TornadoApplication()

    def test_authenticated(self):
        kwargs = {"component": self.application.components['test']}
        handler = MainHandler(self.application, FakeRequest(), **kwargs)
        self.assertFalse(handler.authenticated())
        handler.current_user = "a user"
        self.assertTrue(handler.authenticated())


class TornadoLaucherTestCase(unittest.TestCase):
    """ TornadoLaucher tests
    """

    def test_parameters_none(self):
        """ Test if launcher parameters will be none if not informed
        """
        launcher = FirenadoLauncher()
        self.assertIsNone(launcher.addresses)
        self.assertIsNone(launcher.dir)
        self.assertIsNone(launcher.port)

    def test_parameters_addresses_and_port(self):
        """ Test if launcher parameters were set correctly if informed
        addresses, dir and port
        """
        addresses = "localhost"
        dir = os.path.dirname(os.path.abspath(__file__))
        port = 80
        launcher = FirenadoLauncher(addresses=addresses, dir=dir, port=port)
        self.assertEqual(addresses, launcher.addresses)
        self.assertEqual(dir, launcher.dir)
        self.assertEqual(port, launcher.port)

    def test_parameters_socket(self):
        """ Test if launcher parameters were set correctly if informed dir and
        socket
        """
        dir = os.path.dirname(os.path.abspath(__file__))
        socket = "/tmp/a_socket_file"
        launcher = FirenadoLauncher(dir=dir, socket=socket)
        self.assertEqual(dir, launcher.dir)
        self.assertEqual(socket, launcher.socket)

    def test_load(self):
        chdir_app('tornadoweb')
        launcher = TornadoLauncher()
        launcher.load()
        self.assertTrue(isinstance(launcher.application, TornadoApplication))



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

import firenado.config as _config
import logging
import os
import sys
import tempfile

# Setting root path
ROOT = None
if os.path.abspath(__file__).endswith(".py") or \
        os.path.abspath(__file__).endswith(".pyc"):
    ROOT = os.path.dirname(os.path.abspath(__file__))
else:
    ROOT = os.path.abspath(__file__)
ROOT = os.path.abspath(os.path.join(ROOT, ".."))

# Getting configuration paths and files from the environment
FIRENADO_CONFIG_FILE = None
try:
    FIRENADO_CONFIG_FILE = os.environ['FIRENADO_CONFIG_FILE']
except KeyError:
    FIRENADO_CONFIG_FILE = "firenado"

stack = []

LIB_CONFIG_FILE = os.path.join(ROOT, "conf", FIRENADO_CONFIG_FILE)

# Application file
APP_ROOT_PATH = os.path.join(os.getcwd())
# If FIRENADO_CURRENT_APP_CONFIG_PATH is not set than return current directory
# conf dir
APP_CONFIG_PATH = os.getenv("FIRENADO_CURRENT_APP_CONFIG_PATH",
                            os.path.join(APP_ROOT_PATH, "conf"))
#print(APP_CONFIG_PATH)
APP_CONFIG_FILE = os.path.join(APP_CONFIG_PATH, FIRENADO_CONFIG_FILE)

# If FIRENADO_SYS_CONFIG_PATH is not set than set default sys config path
SYS_CONFIG_PATH = os.getenv("FIRENADO_SYS_CONFIG_PATH",
                            os.path.join(os.sep, "etc", "firenado"))
SYS_CONFIG_FILE = os.path.join(SYS_CONFIG_PATH, FIRENADO_CONFIG_FILE)

HAS_LIB_CONFIG_FILE = False
HAS_SYS_CONFIG_FILE = False
HAS_APP_CONFIG_FILE = False

config_file_extensions = ["yml", "yaml"]
for extension in config_file_extensions:
    if not HAS_LIB_CONFIG_FILE:
        if os.path.isfile("%s.%s" % (LIB_CONFIG_FILE, extension)):
            HAS_LIB_CONFIG_FILE = True
            LIB_CONFIG_FILE = "%s.%s" % (LIB_CONFIG_FILE, extension)
            stack.append(LIB_CONFIG_FILE)
    if not HAS_SYS_CONFIG_FILE:
        if os.path.isfile("%s.%s" % (SYS_CONFIG_FILE, extension)):
            HAS_SYS_CONFIG_FILE = True
            SYS_CONFIG_FILE = "%s.%s" % (SYS_CONFIG_FILE, extension)
            stack.append(SYS_CONFIG_FILE)
    if not HAS_APP_CONFIG_FILE:
        if os.path.isfile("%s.%s" % (APP_CONFIG_FILE, extension)):
            HAS_APP_CONFIG_FILE = True
            APP_CONFIG_FILE = "%s.%s" % (APP_CONFIG_FILE, extension)
            stack.append(APP_CONFIG_FILE)

# Tmp path variable
# TODO: Should I care about windows?
TMP_SYS_PATH = tempfile.gettempdir()
TMP_APP_PATH = TMP_SYS_PATH

# Setting firenado's default variables

apps = {}

# Application section
app = _config.get_app_defaults()

is_multi_app = False
current_app_name = os.environ.get("CURRENT_APP", None)

# Component section
components = {}

# Data section
data = {}
data['connectors'] = {}
data['sources'] = {}

# Logging default configuration
log = {}
log['format'] = None
log['level'] = logging.NOTSET

# Management section
management = {}
management['commands'] = []

# Session section
session = {}
# Default session hiccup time is 10 seconds
# This is the time the callback will hiccup if purge_limit is reached
session['callback_hiccup'] = 10
# Default session callback time is 2 minutes
# This is the time the application will scan for expired sessions
session['callback_time'] = 120
session['enabled'] = False
session['encoder'] = "pickle"
session['encoders'] = {}
session['file'] = {}
session['file']['path'] = ""
session['handlers'] = {}
session['id_generators'] = {}
# Default session life time is 30 minutes or 1800 seconds
# If set to 0 the session will not expire
session['life_time'] = 1800
session['name'] = "FIRENADOSESSID"
session['prefix'] = "firenado:session"
session['purge_limit'] = 500
session['redis'] = {}
session['redis']['data'] = {}
session['redis']['data']['source'] = ""
session['type'] = ""

if HAS_LIB_CONFIG_FILE:
    lib_config = _config.load_yaml_config_file(LIB_CONFIG_FILE)
    _config.process_config(sys.modules[__name__], lib_config)

if HAS_SYS_CONFIG_FILE:
    sys_config = _config.load_yaml_config_file(SYS_CONFIG_FILE)
    _config.process_config(sys.modules[__name__], sys_config)

if HAS_APP_CONFIG_FILE:
    app_config = _config.load_yaml_config_file(APP_CONFIG_FILE)
    _config.process_app_config(sys.modules[__name__], app_config)

# Set logging basic configurations
for handler in logging.root.handlers[:]:
    # clearing loggers, solution from: https://bit.ly/2yTchyx
    logging.root.removeHandler(handler)

logging.basicConfig(level=log['level'], format=log['format'])



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .tornadoweb import TornadoComponent
from cartola import config, sysexits
from datetime import datetime
import logging
from six import iteritems
import sys
import tornado.ioloop

logger = logging.getLogger(__name__)

try:
    from croniter import croniter
except ImportError:
    logger.error("*** DEPENDENCY ERROR ***\n The croniter module isn't "
                 "installed.\n You need to either install firenado with "
                 "extras(all or schedule) or install croniter to use "
                 "firenado.schedule package.\n\n Possible fixes:\n  pip "
                 "install firenado[all]\n  pip install firenado[schedule]\n  "
                 "pip install croniter\n\n You can also add firenado[all] or "
                 "firenado[scheudle] or croniter to you requirements file "
                 "also.\n It's recommended to use firenado with extras "
                 "instead of installing croniter directly to avoid any "
                 "version compatibility error.\n The croniter package"
                 "installed by extras was tested with firenado and no errors "
                 "are expected during the execution.")
    sys.exit(sysexits.EX_FATAL_ERROR)


def next_from_cron(cron):
    """ Return next schedule run offset from now to the time of execution

    :param cron: The cron string
    :return:
    """
    iterator = croniter(cron, datetime.now())
    return iterator.get_next(datetime)


class Scheduler(object):

    def __init__(self, component, **kwargs):
        """
        Scheduler constructor. It will receive a scheduled component and
        loop interval as parameters.

        :param ScheduledTornadoComponent component: The scheduled component
        that owns the scheduler
        owns the scheduler.
        :param int interval:
        """
        self._can_run = False
        self._id = None
        self._jobs = {}
        self._interval = kwargs.get("interval", 1000)
        self._name = None
        self.component = component
        self._periodic_callback = None

    @property
    def id(self):
        return self._id

    @property
    def jobs(self):
        return [job for _, job in iteritems(self._jobs)]

    @property
    def name(self):
        return self._name

    @property
    def can_run(self):
        return self._can_run

    def add_job(self, job):
        logger.info("Adding job %s into the scheduler [id: %s, name: %s]." %
                    (job.id, self.id, self.name))
        self._jobs[job.id] = job

    def get_job(self, job_id):
        return self._jobs.get(job_id)

    def initialize(self, **kwargs):
        self._id = kwargs.get("id")
        self._name = kwargs.get("name")
        logger.info("Initializing scheduler [id: %s, name: %s]." % (
            self.id, self.name))

    def load_jobs(self):
        raise NotImplementedError

    def remove_job(self, job_id):
        logger.info("Removing job %s from the scheduler [id: %s, name: %s]."
                    % (job_id, self.id, self.name))
        job = self.get_job(job_id)
        if job is None:
            return None
        del(self._jobs[job_id])
        return job.id

    def run(self):
        self.load_jobs()
        logger.info("Scheduler [id: %s, name: %s] interval set to %sms." %
                    (self.id, self.name, self._interval))
        self._periodic_callback = tornado.ioloop.PeriodicCallback(
            self._manage_jobs, self._interval)
        self._periodic_callback.start()

    def _manage_jobs(self):
        logger.debug("Scheduler [id: %s, name: %s] managing jobs." %
                    (self.id, self.name))
        logger.debug("Scheduler [id: %s, name: %s] stopping periodic callback."
                     % (self.id, self.name))
        self._periodic_callback.stop()
        for job in self.jobs:
            if not job.already_scheduled:
                logger.info("Job %s from Scheduler [id: %s, name: %s] isn't "
                            "scheduled yet." % (job.hard_id, self.id,
                                                self.name))
                if job.must_schedule:
                    logger.info(
                        "Job %s from Scheduler [id: %s, name: %s] must be "
                        "scheduled to run at %s." % (
                            job.hard_id, self.id, self.name, job.next_run))
                    job.schedule()
            else:
                logger.debug("Job %s from Scheduler [id: %s, name: %s] already"
                             "scheduled." % (job.hard_id, self.id,
                                                self.name))

        logger.debug("Scheduler [id: %s, name: %s] ending of managing jobs." %
                    (self.id, self.name))
        logger.debug("Scheduler [id: %s, name: %s] starting periodic callback."
                     % (self.id, self.name))
        self._periodic_callback.start()


class ConfScheduler(Scheduler):

    def __init__(self, scheduled_component, **kwargs):
        super(ConfScheduler, self).__init__(scheduled_component, **kwargs)
        self._conf = kwargs.get("conf", None)

    def load_jobs(self):
        if "jobs" in self._conf:
            for job_conf in self._conf['jobs']:
                job_id = job_conf.get("id")
                job_resolved_class = None
                job_cron = job_conf.get("cron")
                job_date = job_conf.get("date")
                if 'class' in job_conf:
                    job_resolved_class = config.get_from_string(
                        job_conf['class'])
                if job_resolved_class is None:
                    logger.warning("Firenado Scheduled Job Error:\n    The "
                                   "scheduled class job %s was not found "
                                   "in the file:\n        %s\n    Job id: %s"
                                   "\n    Scheduler: [id: %s, name: %s]"
                                   "\n        Config: %s\n"
                                   "\n    Please fix this issue.\n" %
                                   (job_conf['class'],
                                    self.component.get_complete_config_file(),
                                    job_id, self.id, self.name, job_conf))
                else:
                    if job_id is None:
                        logger.warning(
                            "Firenado Scheduled Job Error:\n    The "
                            "scheduled job id not defined in the "
                            "file:\n        %s"
                            "\n    Scheduler: [id: %s, name: %s]"
                            "\n        Config: %s\n"
                            "\n    Please fix this issue.\n" %
                            (self.component.get_complete_config_file(),
                             self.id, self.name, job_conf))
                    else:
                        if job_cron is None and job_date is None:
                            logger.warning(
                                "Firenado Scheduled Job Error:\n    The "
                                "scheduled job cron or date not defined in "
                                "the file:\n        %s\n    Job id: %s"
                                "\n    Scheduler: [id: %s, name: %s]"
                                "\n        Config: %s\n"
                                "\n    Please define either a cron string or "
                                "date.\n" %
                                (self.component.get_complete_config_file(),
                                 job_id, self.id, self.name, job_conf))
                        else:
                            self.add_job(job_resolved_class(self, **job_conf))

    def initialize(self):
        has_error = False
        if self._conf is None:
            logger.warning("It is not possible to initialize the scheduler "
                           "because the configuration is missing.")
            has_error = True
        if "id" not in self._conf:
            logger.warning("It is not possible to initialize the scheduler "
                           "because id is missing in the configuration "
                           "provided.\n Config: %s" % self._conf)
            has_error = True
        if "name" not in self._conf:
            logger.warning("It is not possible to initialize the scheduler "
                           "because name is missing in the configuration "
                           "provided.\n Config: %s" % self._conf)
            has_error = True

        if not has_error:
            self._can_run = True
            super(ConfScheduler, self).initialize(**self._conf)

        if "interval" in self._conf:
            self._interval = self._conf['interval']


class ScheduledJob(object):

    def __init__(self, scheduler, **kwargs):
        self._scheduler = scheduler
        self._id = kwargs.get('id')
        self._date = kwargs.get('date')
        self._cron = kwargs.get('cron')
        self._periodic_callback = None

    @property
    def id(self):
        return self._id

    @property
    def hard_id(self):
        return "%s:%s:%s" % (self._id, self.__class__.__name__, id(self))

    @property
    def date(self):
        return self._date

    @property
    def cron(self):
        return self._cron

    @property
    def already_scheduled(self):
        return self._periodic_callback is not None

    @property
    def next_run(self):
        return next_from_cron(self.cron)

    @property
    def next_interval(self):
        now = datetime.now()
        delta = self.next_run - now
        return delta.total_seconds() * 1000

    @property
    def must_schedule(self):
        return self.next_interval > 0

    def schedule(self):
        next_interval = self.next_interval
        logger.info("Job %s from Scheduler [id: %s, name: %s] scheduled to run"
                    " at next interval of %sms." % (self.hard_id,
                                                    self._scheduler.id,
                                                    self._scheduler.name,
                                                    self.next_interval))
        self._periodic_callback = tornado.ioloop.PeriodicCallback(
            self._run_job, next_interval)
        self._periodic_callback.start()

    async def _run_job(self):
        """ Run the job
        :return None:
        """
        self._periodic_callback.stop()
        logger.info(
            "Running job %s from Scheduler [id: %s, name: %s]." % (
                self.hard_id, self._scheduler.id, self._scheduler.name))
        future = self.run()
        if future is not None:
            logger.debug(
                "Running job %s from Scheduler [id: %s, name: %s] "
                "asynchronously. " % (self.hard_id, self._scheduler.id,
                                      self._scheduler.name))
            await future
        logger.info(
            "Job %s removed from Scheduler [id: %s, name: %s]" % (
                self.hard_id, self._scheduler.id, self._scheduler.name))
        self._periodic_callback = None
        return

    def run(self):
        pass


class ScheduledTornadoComponent(TornadoComponent):

    def __init__(self, name, application):
        super(ScheduledTornadoComponent, self).__init__(name, application)
        self._interval = 1000
        self._has_conf = False
        self._schedulers = {}

    @property
    def has_schedulers(self):
        return len(self._schedulers) > 0

    @property
    def schedulers(self):
        return [schedule for _, schedule in iteritems(self._schedulers)]

    def get_config_filename(self):
        return "%s_%s" % (self.name, "schedule")

    def get_scheduler(self, name):
        if name in self.schedulers:
            return self.schedulers[name]
        return None

    def _config_schedule_component(self):
        if "interval" in self.schedule_conf():
            self._interval = self.conf['interval']
        logger.info("Scheduled component %s interval set to %sms." %
                    (self.name, self._interval))
        if "schedulers" in self.schedule_conf():
            logger.info("Initializing %s schedulers." % self.name)
            self._initialize_schedulers()
        else:
            logger.warning("No scheduler was defined in the scheduled "
                           "component %s.\n\n%s\n* It is necessary either "
                           "define schedulers or scheduler loaders in the \n*"
                           " %s config file. \n* %s-- Firenado scheduled "
                           "component\n%s\n" %
                           (self.name, "*" * 70,
                            self.get_config_file(), " " * 37, "*" * 70))

    def _initialize_schedulers(self):
        for scheduler_conf in self.conf['schedulers']:
            scheduler = ConfScheduler(self, interval=self._interval,
                                      conf=scheduler_conf)
            scheduler.initialize()
            if scheduler.can_run:
                self._schedulers[scheduler.id] = scheduler
                scheduler.run()

    def schedule_conf(self):
        return self.conf

    def initialize(self):
        if self.has_conf:
            logger.info("Configuration file found. Starting scheduled "
                        "component %s initialization." % self.name)
            self._config_schedule_component()
        else:
            logger.warning("No configuration file was found. Scheduled"
                           "component %s won't initialize")



#!/usr/bin/env python
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

import unittest

import firenado.conf
from firenado.config import get_class_from_config
from tests import chdir_app
from firenado.tornadoweb import TornadoApplication


class FileSessionTestCase(unittest.TestCase):
    """ Case that tests an Firenado application after being loaded from its
    configuration file.
    """

    def setUp(self):
        """ Application configuration file will be read and components will be
        loaded.
        """
        chdir_app('file', 'session')
        self.application = TornadoApplication()
        self.session_handler_config = firenado.conf.session[
            'handlers'][firenado.conf.session['type']]
        self.session_handler_config = firenado.conf.session[
            'handlers'][firenado.conf.session['type']]
        self.session_handler_class = get_class_from_config(
            self.session_handler_config)

    def test_defaults_session_parameters(self):
        """ Checks default session parameters on the configuration
        session section
        """
        self.assertEqual(firenado.conf.session['life_time'], 1800)
        self.assertEqual(firenado.conf.session['callback_time'], 120)

    def test_application_session_handler(self):
        """ Checks if the session handler loaded is the same the session
        handler defined.
        """
        self.assertEqual(
            self.application.session_engine.session_handler.__class__,
            self.session_handler_class
        )

    def test_session_type_file(self):
        """ Checks if test component was loaded correctly by the application
        __init__ method.
        """
        application = TornadoApplication()
        session_handler_config = firenado.conf.session[
            'handlers'][firenado.conf.session['type']]
        session_handler_class = get_class_from_config(session_handler_config)
        self.assertEqual(application.session_engine.session_handler.__class__,
                         session_handler_class)


class RedisSessionTestCase(unittest.TestCase):
    """ Case that tests an Firenado application after being loaded from its
    configuration file.
    """

    def setUp(self):
        """ Application configuration file will be read and components will be
        loaded.
        """
        chdir_app('redis', 'session')

    def test_session_type_redis(self):
        """ Checks if test component was loaded correctly by the application
        __init__ method.
        """
        self.assertEqual(firenado.conf.session['enabled'], True)
        self.assertEqual(firenado.conf.session['type'], 'redis')

    def test_custom_session_parameters(self):
        """ Checks default session parameters on the configuration
        session section
        """
        self.assertEqual(firenado.conf.session['life_time'], 1900)
        self.assertEqual(firenado.conf.session['callback_time'], 40)


class EncodersSessionTestCase(unittest.TestCase):

    def test_pickle_session_encoder(self):
        """ Checks if the pickle session encoder will keep a dict structure
        and values intact after encoding and decoding it.
        """
        from firenado.session import PickeSessionEncoder
        my_dict = {
            'value1': "My value1",
            'value2': {
                'value3': "My value3",
            }
        }
        encoder = PickeSessionEncoder()
        encoded_data = encoder.encode(my_dict)
        decoded_data = encoder.decode(encoded_data)
        self.assertEqual(decoded_data['value1'], my_dict['value1'])
        self.assertEqual(decoded_data['value2']['value3'],
                         my_dict['value2']['value3'])

    def test_json_session_encoder(self):
        """Checks if json session encoder will keep a dict structure and values
        intact after encoding and decoding it."""
        from firenado.session import JsonSessionEncoder
        my_dict = {
            'value1': "My value1",
            'value2': {
                'value3': "My value3",
            }
        }
        encoder = JsonSessionEncoder()
        encoded_data = encoder.encode(my_dict)
        decoded_data = encoder.decode(encoded_data)
        self.assertEqual(decoded_data['value1'], my_dict['value1'])
        self.assertEqual(decoded_data['value2']['value3'],
                         my_dict['value2']['value3'])



#!/usr/bin/env python
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

import os
import six

if six.PY3:
    try:
        import importlib
        reload = importlib.reload
    except AttributeError:
        # PY33
        import imp
        reload = imp.reload

TEST_ROOT = os.path.dirname(os.path.abspath(__file__))
PROJECT_ROOT = os.path.abspath(os.path.join(TEST_ROOT, ".."))


def chdir_fixture_app(app_name, **kwargs):
    dir_name = kwargs.get("dir_name", None)
    suppress_log = kwargs.get("suppress_log", False)
    fixture_root = kwargs.get("fixture_root", "fixtures")
    test_app_dirname = os.path.join(TEST_ROOT, fixture_root, app_name)
    if dir_name is not None:
        test_app_dirname = os.path.join(TEST_ROOT, fixture_root,
                                        dir_name, app_name)
    os.chdir(test_app_dirname)
    import firenado.conf
    if suppress_log:
        import logging
        for handler in logging.root.handlers[:]:
            # clearing loggers, solution from: https://bit.ly/2yTchyx
            logging.root.removeHandler(handler)
    return test_app_dirname


def chdir_app(app_name, dir_name=None):
    """ Change to the application directory located at the resource directory
    for conf tests.

    The conf resources directory is firenado/tests/resources/conf.

    :param app_name: The application name
    """
    import firenado.conf

    test_dirname, filename = os.path.split(os.path.abspath(__file__))
    if dir_name:
        test_app_dirname = os.path.join(test_dirname, 'resources',
                                        dir_name, app_name)
    else:
        test_app_dirname = os.path.join(test_dirname, 'resources', app_name)
    os.chdir(test_app_dirname)
    reload(firenado.conf)



#!/usr/bin/env python
#
# Copyright 2015-2019 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

import unittest

from firenado.data import DataConnectedMixin
from firenado.service import served_by, FirenadoService


class MockServiceDataConnected(FirenadoService):
    """ Serves a data connected method directly.
    When decorating a data connected directly the service must return the
    consumer.
    """
    pass


class MockDataConnected(DataConnectedMixin):
    """ Data connected mock object. This object holds the data sources to be
    used in the test cases.
    """

    def __init__(self):
        self.data_sources['datasource1'] = 'DataSource1'
        self.data_sources['datasource2'] = 'DataSource2'

    @served_by(MockServiceDataConnected)
    def get_service_data_sources_directly(self):
        return self.mock_service_data_connected.get_data_sources()


class MockTestService(FirenadoService):
    """ Service that decorates the instance to be served directly and
    indirectly thought MockTestServiceRecursion.
    When decorating directly data connected and data sources will be returned
    in one interaction.
    When decorating indirectly MockTestServiceRecursion will be used to return
    the data connected instance and data sources.
    """
    pass


class MockTestServiceRecursion(FirenadoService):
    """ Service that decorates the instance to be served and will be used
    to return the data connected and data sources when MockTestService
    delegates to during the recursive test.
    """

    @served_by(MockTestService)
    def get_service_data_sources_recursively(self):
        return self.mock_test_service.get_data_sources()

    @served_by(MockTestService)
    def get_data_connected_recursively(self):
        return self.mock_test_service.data_connected


class ServedByInstance(object):
    """ Class that has methods that will be decorated with the served_by
    decorator.
    """

    def __init__(self, data_connected):
        self.data_connected = data_connected

    @served_by(MockTestService)
    def do_served_by_class(self):
        """
        This method will be decorated with served_by with the class reference
        """
        pass

    @served_by('tests.service_test.MockTestService')
    def do_served_by_string(self):
        """
        This method will be decorated with served_by with the class as string
        """
        pass

    @served_by(MockTestService)
    def get_service_data_connected(self):
        return self.mock_test_service.data_connected

    @served_by(MockTestServiceRecursion)
    def get_service_data_connected_recursively(self):
        return (self.mock_test_service_recursion.
                get_data_connected_recursively())

    @served_by(MockTestService)
    def get_service_data_sources(self):
        """ This method returns the data sources from the data connected
        instance of this class. The method is returned by the service defined
        on the served_by decorator. Here there is no recursion once
        MockTestService is directly serving the ServiceByInstance.
        """
        return self.mock_test_service.get_data_sources()

    @served_by(MockTestServiceRecursion)
    def get_service_data_sources_recursively(self):
        """ Same as get_service_data_sources but the a service will be serving
        the service defined here and then access the data sources
        """
        return (self.mock_test_service_recursion.
                get_service_data_sources_recursively())

    def get_data_connected(self):
        return self.data_connected


class ServiceTestCase(unittest.TestCase):

    def setUp(self):
        """ Setting up an object that has firenado.core.service.served_by
        decorators on some methods.
        """
        self.data_connected_instance = MockDataConnected()
        self.served_by_instance = ServedByInstance(
            self.data_connected_instance)

    def test_served_by_class_reference(self):
        self.assertFalse(hasattr(self.served_by_instance, 'mock_test_service'))
        self.served_by_instance.do_served_by_class()
        self.assertTrue(hasattr(self.served_by_instance, 'mock_test_service'))
        self.assertTrue(isinstance(
            self.served_by_instance.mock_test_service, MockTestService))

    def test_served_by_class_name_string(self):
        self.assertFalse(hasattr(self.served_by_instance, 'mock_test_service'))
        self.served_by_instance.do_served_by_string()
        self.assertTrue(hasattr(self.served_by_instance, 'mock_test_service'))
        self.assertEqual(
            self.served_by_instance.mock_test_service.__class__.__name__,
            MockTestService.__name__)

    def test_data_connected_from_service(self):
        data_connected = self.served_by_instance.get_data_connected()
        self.assertEqual(data_connected, self.data_connected_instance)

    def test_data_connected_from_service_recursively(self):
        data_connected = (self.served_by_instance.
                          get_service_data_connected_recursively())
        self.assertEqual(data_connected, self.data_connected_instance)

    def test_data_connected_from_service_none(self):
        mock_service = MockTestService(None)
        data_connected = mock_service.data_connected
        self.assertIsNone(data_connected)

    def test_get_data_source_from_service(self):
        data_sources = self.served_by_instance.get_service_data_sources()
        self.assertTrue(len(data_sources) == 2)
        self.assertEqual(data_sources['datasource1'], "DataSource1")
        self.assertEqual(data_sources['datasource2'], "DataSource2")

    def test_get_data_source_from_data_connected(self):
        data_sources = (self.data_connected_instance.
                        get_service_data_sources_directly())
        self.assertTrue(len(data_sources) == 2)
        self.assertEqual(data_sources['datasource1'], "DataSource1")
        self.assertEqual(data_sources['datasource2'], "DataSource2")

    def test_get_data_source_from_service_recursively(self):
        data_sources = (self.served_by_instance.
                        get_service_data_sources_recursively())
        self.assertTrue(len(data_sources) == 2)
        self.assertEqual(data_sources['datasource1'], "DataSource1")
        self.assertEqual(data_sources['datasource2'], "DataSource2")

    def test_get_data_source_from_service_none(self):
        mock_service = MockTestService(None)
        data_sources = mock_service.get_data_sources()
        self.assertIsNone(data_sources)



#!/usr/bin/env python
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import firenado.conf
from firenado.management import ManagementCommand
from firenado.management import tasks
from tornado import template
import os

loader = template.Loader(os.path.join(firenado.conf.ROOT,
                                      "management", "templates", "help"))
ManagementCommand(
    "app", "Application related commands", loader.load("app_command_help.txt"),
    category="Firenado", sub_commands=[
        ManagementCommand("install", "Install a Firenado application", "",
                          tasks=tasks.InstallProjectTask),
        ManagementCommand("run", "Runs a Firenado application", "",
                          tasks=tasks.RunApplicationTask)
    ])
ManagementCommand(
    "proj(ect)", "Project related commands",
    loader.load("project_command_help.txt"), category="Firenado",
    sub_commands=[
        ManagementCommand("init", "Initiates a new Firenado project", "",
                          tasks=tasks.CreateProjectTask)
    ])
ManagementCommand(
    "rand(om)", "Random related commands", loader.load(
        "random_command_help.txt"),
    category="Firenado", sub_commands=[
        ManagementCommand("string", "Generates a random string", "",
                          tasks=tasks.GenerateRandomStringTask),
        ManagementCommand("uuid", "Generates a random uuid", "",
                          tasks=tasks.GenerateUuidTask)
    ])



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from behave import given, when, then
from behave.api.async_step import async_run_until_complete
from tests import PROJECT_ROOT, chdir_fixture_app
from firenado.launcher import ProcessLauncher
from tornado import gen
from tornado.httpclient import AsyncHTTPClient
import sys


@given("We launch {application} application using process launcher at {port} "
       "port")
@async_run_until_complete
async def step_we_launch_application_using_process_launcher(context,
                                                            application, port):
    application_dir = chdir_fixture_app(application, suppress_log=True)
    context.launcher = ProcessLauncher(
        dir=application_dir, path=PROJECT_ROOT, port=port, logfile=sys.stderr)
    context.launcher.load()
    await context.launcher.launch()
    await gen.sleep(1)
    context.tester.assertTrue(context.launcher.is_alive())


@when("The application is running correctly at {port} port")
@async_run_until_complete
async def step_we_launch_application_using_process_launcher(context, port):
    http_client = AsyncHTTPClient()
    try:
        response = await http_client.fetch("http://localhost:%s" %
                                           port)
    except Exception as e:
        print("Error: %s" % e)
        context.tester.assertTrue(False)
    else:
        context.tester.assertEquals(b"IndexHandler output", response.body)
        context.tester.assertTrue(context.launcher.is_alive())


@then("We shutdown the process launcher")
def step_we_shutdown_the_process_launcher(context):
    context.tester.assertTrue(context.launcher.is_alive())
    context.launcher.shutdown()


@then("The application stops successfully")
def step_the_application_stops_successfully(context):
    context.tester.assertFalse(context.launcher.is_alive())



#!/usr/bin/env python
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from sqlalchemy import inspect, text
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

# Named as sqlalchemy_util to avoid conflict with the original sqlalchemy
# module.

Base = declarative_base()
Session = sessionmaker()
logger = logging.getLogger(__name__)


def base_to_dict(base, columns=None):
    """ Returns a dict inherited from Base. If keys is provided it will only
    add to the dict the columns provided.
    :param base: An orm object inherited from Base
    :param columns: Columns to be added to the dict
    :return dict: A dictionary representing the Base object.
    """
    if columns is None:
        return {c.key: getattr(base, c.key)
                for c in inspect(base).mapper.column_attrs}
    return {c.key: getattr(base, c.key)
            for c in inspect(base).mapper.column_attrs if c.key in columns}


def run_script(script_path, session, handle_command=None, handle_line=None):
    """ Run a script file using a valid sqlalchemy session.

    Based on https://bit.ly/2CToAhY.
    See also sqlalchemy transaction control: https://bit.ly/2yKso0A

    :param script_path: The path where the script is located
    :param session: A sqlalchemy session to execute the sql commands from the
    script
    :param handle_command: Function to handle a valid command
    :param handle_line: Function to handle a valid line
    :return:
    """
    logger.debug("Opening script %s." % script_path)
    with open(script_path, "r") as stream:
        sql_command = ""
        for line in stream:
            # Ignore commented lines
            if not line.startswith("--") and line.strip("\n"):
                # Append line to the command string
                if handle_line is not None:
                    logger.debug("Calling the handle line function for: "
                                 "%s." % line)
                    line = handle_line(line)
                sql_command = "%s%s" % (sql_command, line.strip("\n"))
                # If the command string ends with ";", it is a full statement
                if sql_command.endswith(";"):
                    # Try to execute statement and commit it
                    try:
                        if handle_command is not None:
                            logger.debug("Calling the handle command function "
                                         "for: %s." % sql_command)
                            sql_command = handle_command(sql_command)
                        session.execute(text(sql_command))
                    # Assert in case of error
                    except Exception as e:
                        session.rollback()
                        raise e
                    # Finally, clear command string
                    finally:
                        sql_command = ""
    session.commit()



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

from cartola import fs, security
import firenado.conf
from firenado.config import get_class_from_config

import functools
import logging
import os
import tornado
import tornado.ioloop

logger = logging.getLogger(__name__)


class SessionEngine(object):
    """SessionEngine provides session support to an application.
    """

    def __init__(self, session_aware_instance):
        self.session_aware_instance = None
        self.session_handler = None
        self.session_callback = None
        self.callback_time = None
        self.callback_hiccup = firenado.conf.session['callback_hiccup'] * 1000

        # TODO: By the way session could be disabled. How do we 
        # handle that?
        # TODO: check if session type exists. Maybe disable it if type is not
        # defined. We need to inform the error here
        if firenado.conf.session['enabled']:
            # Transforming session callback time to milliseconds.
            self.callback_time = firenado.conf.session['callback_time'] * 1000
            logging.debug("Session periodic callback will be set with time of "
                          "%sms." % self.callback_time)
            session_handler_class = get_class_from_config(
                firenado.conf.session['handlers'][
                    firenado.conf.session['type']]
            )
            logging.debug("Setting session handler %s.%s." %
                          (session_handler_class.__module__,
                           session_handler_class.__name__)
                          )
            self.session_aware_instance = session_aware_instance
            self.session_handler = session_handler_class(self)
            self.session_callback = tornado.ioloop.PeriodicCallback(
                self.session_handler.purge_expired_sessions,
                self.callback_time
            )
            self.session_handler.set_settings({})
            self.session_handler.configure()

            # Starting session periodic callback
            self.session_callback.start()
            logging.debug("Session periodic callback started by the engine.")
            encoder_class = get_class_from_config(
                firenado.conf.session['encoders'][
                    firenado.conf.session['encoder']
                ]
            )
            self.session_encoder = encoder_class()

    def get_session(self, request_handler):
        """Returns a valid session object. This session is handler by the
        session handler defined on the application configuration. """
        if firenado.conf.session['enabled']:
            session = Session(self)
            cookie_created_on_request = False
            session_id = SessionHandler.get_session_id_cookie(request_handler)
            if session_id is None:
                cookie_created_on_request = True
                session = self.__renew_session(request_handler)
            else:
                session.id = session_id
            if not self.session_handler.is_session_stored(session_id):
                if not cookie_created_on_request:
                    # Regenerating the session id. Because the
                    # cookie was not created during at the same request
                    # the file is being created.
                    cookie_created_on_request = True
                    session = self.__renew_session(request_handler)
            if not cookie_created_on_request:
                session_data = self.decode_session_data(
                    self.session_handler.read_stored_session(session_id))
                session = Session(self, session_data, session_id)
            return session

    def store_session(self, request_handler):
        """Sends the session data to be stored by the session handler defined
        on the application configuration. """
        if firenado.conf.session['enabled']:
            session_id = request_handler.session.id
            # If session was destroyed than we gonna handle it differently
            # If session id is None than ignored (it is probably a redirect
            # leftover from security)
            if session_id is not None:
                if not request_handler.session.is_destroyed():
                    encoded_session_data = self.encode_session_data(
                        request_handler.session.get_data())
                    if request_handler.session.is_changed():
                        self.session_handler.write_stored_session(
                            session_id,
                            encoded_session_data
                        )
                else:
                    # Generating a new session
                    logger.debug("Dispatching session %s destruction to the "
                                 "session handler." % session_id)
                    self.session_handler.destroy_stored_session(session_id)

    def encode_session_data(self, data):
        return self.session_encoder.encode(data)

    def decode_session_data(self, data):
        return self.session_encoder.decode(data)

    def get_session_aware_instance(self):
        return self.session_aware_instance

    def set_purge_normal(self):
        self.session_callback.callback_time = self.callback_time
        logger.debug("No hiccups. Callback in %sms." % self.callback_time)

    def set_purge_hiccup(self):
        self.session_callback.callback_time = self.callback_hiccup
        logger.warning("Purge hiccup in %sms." % self.callback_hiccup)

    def __renew_session(self, request_handler):
        if firenado.conf.session['enabled']:
            session = Session(self)
            session_id = SessionHandler.create_session_id_cookie(
                request_handler)
            session.id = session_id
            self.session_handler.create_session(
                session_id,
                self.encode_session_data(session.get_data())
            )
            return session
        return None


class SessionEnginedMixin(object):
    """
    This mixin includes a configured session engine to an entity.
    The engine will handle session reads and writes triggered by
    session engined entity.

    Example:
    >>> class MyApplication(firenado.tornadoweb.TornadoApplication,
    >>>                     SessionEnginedMixin):

    Refer to SessionEngine documentation in order to know which methods are
    available.
    """

    @property
    def session_engine(self):
        """ Returns a SessionEngine instance """
        return create_session_engine(self, '__session_engine', SessionEngine)


class Session(object):

    def __init__(self, engine, data=None, sess_id=None):
        self.__engine = engine
        self.id = sess_id
        self.__data = {} if data is None else data
        self.__destroyed = False
        self.__changed = False

    def clear(self):
        """ Clear all data stored into the session. This is not 
        renewing/creating a new session id. If you want that use 
        session.destroy() """
        self.__data.clear()
        self.__changed = True

    def destroy(self, request_handler):
        """ Clearing session data and marking the session to be
        renewed at the end of the request. """
        self.clear()
        self.__destroyed = True
        self.__engine.store_session(request_handler)

    def get(self, key):
        """ Returns the value from the session data by a given key """
        self.__lock_if_destroyed()
        if self.has(key):
            return self.__data[key]
        return None

    def get_data(self):
        self.__lock_if_destroyed()
        return self.__data.copy()

    def has(self, key):
        self.__lock_if_destroyed()
        """ Returns if session data has some data stored by a given key. """
        return key in self.__data

    def delete(self, key):
        self.__lock_if_destroyed()
        if key in self.__data:
            del self.__data[key]
            self.__changed = True

    def is_destroyed(self):
        """ Returns is the session is marked to be destroyed 
        during session storing procedure. """
        return self.__destroyed

    def is_changed(self):
        return self.__changed

    def __lock_if_destroyed(self):
        if self.is_destroyed():
            raise SessionDestroyedError()

    def set(self, key, value):
        """ Set a value into the session data labeled by a given key """
        self.__lock_if_destroyed()
        self.__data[key] = value
        self.__changed = True


class SessionDestroyedError(tornado.web.HTTPError):

    def __init__(self, status_code, log_message=None, *args, **kwargs):
        message = "The session was destroyed. It is necessary a renew the " \
                  "session before use it again."
        super(SessionDestroyedError, self).__init__(
            505, message, *args, **kwargs)


class SessionHandler(object):

    def __init__(self, engine):
        self.engine = engine
        self.settings = {}

    def create_session(self, session_id, data):
        pass

    def read_stored_session(self, session_id):
        pass

    def write_stored_session(self, session_id, data):
        pass

    def destroy_stored_session(self, session_id):
        pass

    def purge_expired_sessions(self):
        pass

    @staticmethod
    def create_session_id_cookie(request_handler):
        session_id = SessionHandler.__generate_session_id()
        if 'cookie_secret' in request_handler.application.settings:
            settings = SessionHandler.__session_id_cookie_settings(secret=True)
            expires_days = settings.pop('expires_days')
            request_handler.set_secure_cookie(
                firenado.conf.session['name'], session_id,
                expires_days=expires_days)
        else:
            request_handler.set_cookie(
                name=firenado.conf.session['name'], value=session_id,
                **SessionHandler.__session_id_cookie_settings())
        return session_id

    @staticmethod
    def get_session_id_cookie(request_handler):
        if 'cookie_secret' in request_handler.application.settings:
            cookie_id = request_handler.get_secure_cookie(
                firenado.conf.session['name'])
            if cookie_id is not None:
                return cookie_id.decode()
            return cookie_id
        return request_handler.get_cookie(firenado.conf.session['name'])

    def is_session_stored(self, session_id):
        pass

    def configure(self):
        pass

    def set_settings(self, settings):
        self.settings = settings

    @staticmethod
    def __session_id_cookie_settings(secret=False):
        """ Defines some settings to be used with the session id cookie. """
        cookie_settings = {}
        cookie_settings.setdefault('expires', None)
        cookie_settings.setdefault('expires_days', None)
        if secret:
            cookie_settings['expires_days'] = 30
        return cookie_settings

    @staticmethod
    def __generate_session_id():
        """
        Retrieves the session id generator function from the configuration.

        It is possible for a developer create a new session id generator
        function and and use it here.

        Returns:
            A probably unique session id.
        """
        session_id_generator = get_class_from_config(
            firenado.conf.session['id_generators'][
                firenado.conf.app['session']['id_generator']
            ], "function"
        )
        return session_id_generator()


def create_session_engine(obj, session_engine_attribute, session_engine_class):
    if not hasattr(obj, session_engine_attribute):
        session_engine_instance = session_engine_class(obj)
        setattr(obj, session_engine_attribute, session_engine_instance)
    return getattr(obj, session_engine_attribute)


def read(method):
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        if firenado.conf.session['enabled']:
            session = self.application.session_engine.get_session(self)
            self.session = session
            logging.debug("Reading session %s." % self.session.id)
        return method(self, *args, **kwargs)
    return wrapper


def write(method):
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        retval = method(self, *args, **kwargs)
        if firenado.conf.session['enabled']:
            if self.session is None:
                logging.error("The handler session is None. Something wrong"
                              " happened during the handler execution. The"
                              " current handler status is: %s." %
                              self.get_status())
            else:
                logging.debug("Writing session %s." % self.session.id)
                self.application.session_engine.store_session(self)
        return retval
    return wrapper


class FileSessionHandler(SessionHandler):
    """
    Session handler that deals with file data stored in files.

    The FileSessionHandler blocks tornado requests as long we are reading and
    writing files. We will try to minimize the blocking effect but by now don't
    use this for high traffic sites.
    """

    path = None

    def configure(self):
        self.life_time = int(firenado.conf.session['life_time'])
        if os.path.exists(firenado.conf.session['file']['path']):
            self.path = firenado.conf.session['file']['path']
        else:
            # TODO: Need to think about this. I don't know if this is a
            # good behaviour. Maybe just crash the app here
            # or disable the session with a good warning.
            self.path = firenado.conf.TMP_APP_PATH

    def create_session(self, session_id, data):
        # TODO: What could possibly go wrong here? Let's handle it!
        if os.path.exists(firenado.conf.session['file']['path']):
            session_file = os.path.join(self.path,
                                        self.__get_filename(session_id))
            fs.touch(session_file)
            self.write_stored_session(session_id, data)

    def read_stored_session(self, session_id):
        import binascii
        session_file = os.path.join(self.path, self.__get_filename(session_id))
        timed_data = fs.read(session_file)
        return binascii.unhexlify(timed_data.split("--")[0])

    def write_stored_session(self, session_id, data):
        import binascii
        import time
        timed_data = "%s--%s" % (binascii.hexlify(data).decode("ascii"),
                                 int(time.time()))
        session_file = os.path.join(self.path, self.__get_filename(session_id))
        fs.write(session_file, timed_data)

    def destroy_stored_session(self, session_id):
        logger.debug("Destroying session %s." % session_id)
        try:
            session_file = os.path.join(
                self.path,
                self.__get_filename(session_id)
            )
            os.remove(session_file)
            logger.debug("Session %s destroyed." % session_id)
        except OSError:
            # TODO Why we are deleting the session file twice?
            pass

    def purge_expired_sessions(self):
        import time
        """ A file session contains a data separated by --
        The first part is the hexadecimal representation of the encoded session
        data. Second is the epoch of the last write.
        If current epoch - file epoch is lower than the session life time then
        we need to delete this file.
        """
        logger.debug("File handler looking for expired sessions.")
        self.engine.session_callback.stop()
        logging.debug("Session periodic callback stopped by the file "
                      "handler.")
        purge_count = 0
        purge_hiccup = False
        for dirname, dirnames, filenames in os.walk(self.path):
            for filename in filenames:
                file_path = os.path.join(self.path, filename)
                sess_id = filename.split(".")[0].split("_")[-1]
                timed_data = fs.read(file_path)
                last_write = timed_data.split("--")[1]
                age = int(time.time()) - int(last_write)
                if age > self.life_time:
                    logging.debug("Session %s is expired. Removing file "
                                  "from the session path." % sess_id)
                    os.remove(file_path)
                    purge_count += 1
            if purge_count == firenado.conf.session['purge_limit']:
                purge_hiccup = True
                logger.warning(
                    "Expired 500 sessions. Exiting the call and waiting for "
                    "purge hiccup."
                )
                break
        if purge_hiccup:
            self.engine.set_purge_hiccup()
        else:
            self.engine.set_purge_normal()
        self.engine.session_callback.start()
        logging.debug("Session periodic callback resumed by the file "
                      "handler.")

    def is_session_stored(self, session_id):
        return os.path.isfile(os.path.join(self.path, self.__get_filename(
            session_id)))

    def __get_filename(self, session_id):
        return 'firenado_%s.sess' % session_id


class RedisSessionHandler(SessionHandler):
    """
    Session handler that deals with file data stored  in a redis database.
    """
    def __init__(self, engine):
        SessionHandler.__init__(self, engine)
        self.data_source = None

    def configure(self):
        self.life_time = firenado.conf.session['life_time']
        self.data_source = (
            self.engine.get_session_aware_instance().get_data_source(
                firenado.conf.session['redis']['data']['source']
            )
        )

    def create_session(self, session_id, data):
        self.write_stored_session(session_id, data)

    def read_stored_session(self, session_id):
        key = self.__get_key(session_id)
        self.data_source.get_connection().expire(key, self.life_time)
        return self.data_source.get_connection().get(key)

    def write_stored_session(self, session_id, data):
        key = self.__get_key(session_id)
        self.data_source.get_connection().set(key, data)
        self.data_source.get_connection().expire(key, self.life_time)

    def destroy_stored_session(self, session_id):
        logger.debug("Destroying session %s." % session_id)
        key = self.__get_key(session_id)
        self.data_source.get_connection().delete(key)
        logger.debug("Session %s destroyed." % session_id)

    def purge_expired_sessions(self):
        """ On Redis we don't destroy expired sessions per-se.
        If a session has no ttl we just reset an expiration value to it.
        """
        logger.debug("Redis handler looking for sessions without ttl.")
        self.engine.session_callback.stop()
        logging.debug("Session periodic callback stopped by the redis "
                      "handler.")
        keys = self.data_source.get_connection().keys(self.__get_key("*"))
        purge_count = 0
        purge_hiccup = False
        for key in keys:
            ttl = self.data_source.get_connection().ttl(key)
            if ttl == -1:
                logger.warning(
                    "Session %s without ttl. Setting expiration now." % key
                )
                self.data_source.get_connection().expire(key, self.life_time)
                purge_count += 1
                if purge_count == firenado.conf.session['purge_limit']:
                    purge_hiccup = True
                    logger.warning(
                        "Set ttl to 500 sessions. Exiting the call and waiting"
                        " for purge hiccup."
                    )
                    break
        if purge_hiccup:
            self.engine.set_purge_hiccup()
        else:
            self.engine.set_purge_normal()
        self.engine.session_callback.start()
        logging.debug("Session periodic callback resumed by the redis "
                      "handler.")

    def is_session_stored(self, session_id):
        key = self.__get_key(session_id)
        return self.data_source.get_connection().get(key) is not None

    def __get_key(self, session_id):
        if firenado.conf.app['id'] is not None:
            return '%s:%s:%s' % (
                firenado.conf.session['prefix'],
                firenado.conf.app['id'],
                session_id
            )
        return '%s:%s' % (
            firenado.conf.session['prefix'],
            session_id
        )


class SessionEncoder(object):

    def encode(self, data):
        return data

    def decode(self, data):
        return data


class PickeSessionEncoder(SessionEncoder):

    def encode(self, data):
        import pickle
        return pickle.dumps(data, 2)

    def decode(self, data):
        import pickle
        return pickle.loads(data)


class JsonSessionEncoder(SessionEncoder):

    def encode(self, data):
        from tornado import escape
        return escape.json_encode(data)

    def decode(self, data):
        from tornado import escape
        return escape.json_decode(data)


def generate_session_id():
    """
    Default firenado session id generator

    Returns:
        A random string containing digits, upper and lower characters
    """
    return security.random_string(64)



#!/usr/bin/env python
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

import unittest
from tests import (components_test, conf_test, config_test, security_test,
                   service_test, session_test, tornadoweb_test)
from tests.util import sqlalchemy_util_test, url_util_test


def suite():
    testLoader = unittest.TestLoader()
    alltests = unittest.TestSuite()
    alltests.addTests(testLoader.loadTestsFromModule(components_test))
    alltests.addTests(testLoader.loadTestsFromModule(conf_test))
    alltests.addTests(testLoader.loadTestsFromModule(config_test))
    alltests.addTests(testLoader.loadTestsFromModule(security_test))
    alltests.addTests(testLoader.loadTestsFromModule(service_test))
    alltests.addTests(testLoader.loadTestsFromModule(session_test))
    alltests.addTests(testLoader.loadTestsFromModule(tornadoweb_test))
    alltests.addTests(testLoader.loadTestsFromModule(sqlalchemy_util_test))
    alltests.addTests(testLoader.loadTestsFromModule(url_util_test))
    return alltests


if __name__ == "__main__":
    runner = unittest.TextTestRunner(verbosity=3)
    result = runner.run(suite())
    if not result.wasSuccessful():
        exit(2)



#!/usr/bin/env python
#
# Copyright 2015-2019 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
from gettext import gettext as _


class FirenadoArgumentError(Exception):
    """ An error thrown while parsing arguments with FirenadoArgumentParser.
    """
    pass


class FirenadoArgumentParser(argparse.ArgumentParser):
    """ Argument parser that trows an exception leaving stderr untouched.
    """

    def error(self, message):
        args = {'message': message}
        message = _("%(message)s") % args
        raise FirenadoArgumentError(message)



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cartola import fs
import firenado.conf
from firenado.management import ManagementTask
from firenado.util import argparse_util
import logging
import os
from six import iteritems
import sys

logger = logging.getLogger(__name__)


class CreateProjectTask(ManagementTask):
    """
    Creates a new project from scratch
    """
    def run(self, namespace):
        from tornado import template
        if len(sys.argv) > 2:
            module = namespace.module
            component = module.replace(".", " ").title().replace(" ", "")
            project_name = module.replace(".", "_").lower()
            project_directory = fs.create_module(module, os.getcwd())
            #TODO: Check if project exists
            #TODO: If doesn't exists create project
            #TODO: If exists throw an error
            loader = template.Loader(os.path.join(firenado.conf.ROOT,
                                                  "management", "templates",
                                                  "project"))
            project_init_content = loader.load("app.py.txt").generate(
                project_name=project_name, module=module, component=component)
            # Generating application firenado component and handlers
            fs.write(os.path.join(project_directory, "__init__.py"), "")
            fs.write(os.path.join(project_directory, "app.py"),
                        project_init_content.decode(sys.stdout.encoding))
            handlers_file_name = os.path.join(project_directory, "handlers.py")
            fs.touch(handlers_file_name)
            project_handlers_content = loader.load("handlers.py.txt").generate(
                handlers=["Index"])
            fs.write(handlers_file_name,
                        project_handlers_content.decode(sys.stdout.encoding))
            # Generating configuration
            project_conf_directory = os.path.join(project_directory, "conf")
            os.mkdir(project_conf_directory)
            project_conf_file = os.path.join(project_conf_directory,
                                             "firenado.yml")
            fs.touch(project_conf_file)
            project_init_content = loader.load("firenado.yml.txt").generate(
                app_name=project_name, module=module, component=component)
            fs.write(project_conf_file,
                        project_init_content.decode(sys.stdout.encoding))
        else:
            #TODO: This thing has to go. The parameter validation should be
            #TODO: executed by the argument parser.
            loader = template.Loader(os.path.join(
                firenado.conf.ROOT, "management", "templates", "help"))
            help_message = loader.load("init_command_help.txt").generate()

    def add_arguments(self, parser):
        """
        Add module argument to the command parser.

        :param parser: The current parser validating the command holding this
        task.
        """
        parser.add_argument("module", help="The project module")

    def get_error_message(self, parser, exception):
        """

        :param parser:
        :param argparse_util.FirenadoArgumentError exception:
        :return:
        """
        return str(exception)


class InstallProjectTask(ManagementTask):
    """ Triggers the install method of all components registered in the
    application.
    """
    def run(self, namespace):
        # TODO: Resolve module if doesn't exists
        if firenado.conf.app['pythonpath']:
            sys.path.append(firenado.conf.app['pythonpath'])
        # TODO This should consider the type of application being handled by
        # Firenado.
        from firenado.tornadoweb import TornadoApplication
        application = TornadoApplication()
        for key, component in iteritems(application.components):
            component.install()


class RunApplicationTask(ManagementTask):

    def add_arguments(self, parser):
        parser.add_argument("-a", "--addresses", default=None)
        parser.add_argument("-A", "--app", default=None)
        parser.add_argument("-d", "--dir", default=None)
        parser.add_argument("-p", "--path", default=None)
        parser.add_argument("-P", "--port", type=int)
        parser.add_argument("-s", "--socket", default=None)

    """ Runs a Firenado Tornado Application based
    on the it's project configuration
    """
    def run(self, namespace):
        #TODO throw a custom error when type is not found
        from firenado.config import get_class_from_config
        parameters = {}
        if namespace.app is not None:
            parameters['app'] = namespace.app
        if namespace.dir is not None:
            parameters['dir'] = namespace.dir
        if namespace.path is not None:
            parameters['path'] = namespace.path
        if namespace.socket is None:
            if namespace.addresses is not None:
                parameters['addresses'] = namespace.addresses.split(",")
            if namespace.port is not None:
                parameters['port'] = namespace.port
        else:
            parameters['socket'] = namespace.socket
        app_type = firenado.conf.app['types'][firenado.conf.app['type']]
        launcher = get_class_from_config(app_type['launcher'])(**parameters)
        launcher.load()
        launcher.launch()


class GenerateRandomStringTask(ManagementTask):
    """ Generates a random string to serve as the cookie secret for an
    application.
    """

    def add_arguments(self, parser):
        parser.add_argument("size", nargs='?', default=64, type=int)

    def run(self, namespace):
        from cartola import security
        size = namespace.size
        if size > 1000000:
            logger.error("The size {} has reached this command "
                         "limit.".format(size))
            sys.exit(1)
        logger.debug("Displaying a random string with size {}".format(size))
        print(security.random_string(size))
        sys.exit(0)


class GenerateUuidTask(ManagementTask):
    """ Generates an uuid4 string
    """
    def run(self, namespace):
        from uuid import uuid4
        logger.debug("Displaying a random uuid4 string.")
        print(uuid4())
        sys.exit(0)



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

from cartola import fs
import tornado.web
import firenado.conf
from . import data
from . import session
from . import uimodules
from .config import get_class_from_config, load_yaml_config_file
import inspect
import logging
import os
from six import iteritems, string_types
from tornado.escape import json_encode
from tornado.httpclient import HTTPRequest
from tornado.template import Loader
import tornado.websocket

logger = logging.getLogger(__name__)


def get_request(url, **kwargs):
    """ Return a HTTPRequest to help with AsyncHTTPClient and HTTPClient
    execution. The HTTPRequest will use the provided url combined with path
    if provided. The HTTPRequest method will be GET by default and can be
    changed if method is informed.
    If form_urlencoded is defined as True a Content-Type header will be added
    to the request with application/x-www-form-urlencoded value.

    :param str url: Base url to be set to the HTTPRequest
    :key form_urlencoded: If the true will add the header Content-Type
    application/x-www-form-urlencoded to the form. Default is False.
    :key method: Method to be used by the HTTPRequest. Default it GET.
    :key path: If informed will add the path to the base url informed. Default
    is None.
    :return HTTPRequest:
    """
    method = kwargs.get("method", "GET")
    path = kwargs.get("path", None)
    form_urlencoded = kwargs.get("form_urlencoded", False)
    if path is not None:
        if not url.endswith("/"):
            url = "%s/" % url
        url = "%s%s" % (url, path)
    request = HTTPRequest(url, method=method)
    if form_urlencoded:
        request.headers.add("Content-Type",
                            "application/x-www-form-urlencoded")
    return request


class TornadoApplication(tornado.web.Application, data.DataConnectedMixin,
                         session.SessionEnginedMixin):
    """ Firenado basic Tornado application.
    """

    def __init__(self, default_host="", transforms=None, **settings):
        logger.debug('Wiring application located at %s.' %
                     firenado.conf.APP_ROOT_PATH)
        self.components = {}
        settings.update(firenado.conf.app['settings'])
        handlers = []
        ui_modules = []
        data.configure_data_sources(firenado.conf.app['data']['sources'], self)
        self.__load_components()
        for key, component in iteritems(self.components):
            component_handlers = component.get_handlers()
            for i in range(0, len(component_handlers)):
                if issubclass(
                    component_handlers[i][1], TornadoHandler
                ) or issubclass(
                    component_handlers[i][1], TornadoWebSocketHandler
                ):
                    if len(component_handlers[i]) < 3:
                        component_handlers[i] = (
                            component_handlers[i][0],
                            component_handlers[i][1],
                            {'component': component}
                        )
                    else:
                        component_handlers[i][1].component = component
            handlers = handlers + component_handlers
            # Adding component ui modules to the application ui modules list
            ui_modules.append(uimodules)
            if component.get_ui_modules():
                ui_modules.append(component.get_ui_modules())
        if firenado.conf.app['component']:
            if firenado.conf.app['static_path']:
                if os.path.isabs(firenado.conf.app['static_path']):
                    settings['static_path'] = firenado.conf.app['static_path']
                else:
                    settings['static_path'] = os.path.join(
                            self.components[firenado.conf.app[
                                'component']].get_component_path(),
                            firenado.conf.app['static_path'])
            else:
                settings['static_path'] = os.path.join(
                        self.components[
                            firenado.conf.app[
                                'component']].get_component_path(), 'static')
        else:
            settings['static_path'] = os.path.join(
                    os.path.dirname(__file__), "static")
        static_url_prefix = firenado.conf.app['static_url_prefix']
        if static_url_prefix != "/":
            static_url_prefix = "%s/" % static_url_prefix
        settings['static_url_prefix'] = static_url_prefix
        if len(ui_modules) > 0:
            settings['ui_modules'] = ui_modules
        if firenado.conf.app['url_root_path'] is not None:
            from .util.url_util import rooted_path
            for idx, handler in enumerate(handlers):
                handler_list = list(handler)
                handler_list[0] = rooted_path(
                    firenado.conf.app['url_root_path'], handler_list[0])
                handlers[idx] = tuple(handler_list)
        tornado.web.Application.__init__(self, handlers=handlers,
                                         default_host=default_host,
                                         transforms=transforms, **settings)
        logger.debug("Checking if session is enabled.")
        if firenado.conf.session['enabled']:
            logger.debug("Session is enabled. Starting session engine.")
            self.session_engine
        else:
            logger.debug("Session is disabled.")

    def get_app_component(self):
        return self.components[firenado.conf.app['component']]

    def __load_components(self):
        """ Loads all enabled components registered from the components
        config section.
        """
        for key, value in iteritems(firenado.conf.components):
            if value['enabled']:
                component_class = get_class_from_config(value)
                self.components[key] = component_class(key, self)
                if self.components[key].get_config_file():
                    filename = self.components[key].get_complete_config_file()
                    if filename is not None:
                        self.components[key].conf = load_yaml_config_file(
                            filename)
                        self.components[key].process_config()
                        self.components[key].has_conf = True
                    else:
                        logger.debug("Failed to find the file for the "
                                     "component %s at %s. Component's "
                                     "filename returned is %s." % (
                                        key, firenado.conf.APP_CONFIG_PATH,
                                        self.components[key].get_config_file())
                                     )
        # Initializing enabled components after the load.
        for key, value in iteritems(firenado.conf.components):
            if value['enabled']:
                self.components[key].initialize()


class TornadoComponent(object):
    """ Firenado applications are organized in components. A component could be
    an application or something that can be distributed as an add-on or a
    plugin.
    """

    def __init__(self, name, application):
        self.name = name
        self.application = application
        self.conf = {}
        self._has_conf = False
        self.plugins = dict()

    def after_request(self, handler):
        """ Add a logic to be executed after all component's handlers
        execution.
        """
        pass

    def before_request(self, handler):
        """ Add a logic to be executed before all component's handler
        execution.
        """
        pass

    def is_current_app(self):
        if not firenado.conf.is_multi_app:
            return True
        else:
            if firenado.conf.current_app_name == self.name:
                return True
        return False

    @property
    def has_conf(self):
        return self._has_conf

    @has_conf.setter
    def has_conf(self, value):
        self._has_conf = value

    def get_handlers(self):
        """ Returns handlers being added by the component to the application.
        :return: A list of handlers the component provides.
        """
        return []

    def get_ui_modules(self):
        """ Returns uimodules the component provides to the application.
        It could be just a module, a list or a dictionary of modules.
        :return: Uimodules the component provides.
        """
        return None

    def get_component_path(self):
        """ Returns the component path.
        """
        return os.path.abspath(os.path.dirname(
            inspect.getfile(self.__class__)))

    def get_config_filename(self):
        return None

    def get_config_file(self):
        filename = self.get_config_filename()
        if filename is not None:
            return filename
        return None

    def get_complete_config_file(self):
        """ Return the config file with the correct extension, if
        get_config_file has no extension.

        :return str: The config file with extension
        """
        if fs.file_has_extension(self.get_config_file()):
            if os.path.isfile(self.get_config_file()):
                return os.path.join(firenado.conf.APP_CONFIG_PATH,
                                    self.get_config_file())
        config_file_extensions = ['yml', 'yaml']
        for extension in config_file_extensions:
            candidate_filename = "%s.%s" % (self.get_config_file(), extension)
            if os.path.isfile(os.path.join(
                    firenado.conf.APP_CONFIG_PATH, candidate_filename)):
                return os.path.join(firenado.conf.APP_CONFIG_PATH,
                                    candidate_filename)
        return None

    def get_template_path(self):
        """ Returns the path that holds the component's templates.
        """
        return os.path.join(os.path.abspath(os.path.dirname(
            inspect.getfile(self.__class__))), 'templates')

    def initialize(self):
        """ If you want to add logic while the component is initializing
        please overwrite this method.
        """
        pass

    def install(self):
        """ Firenado handles an application installation looping thought all
        components and triggering the install method of them.
        If
        """
        pass

    def process_config(self):
        """ To process your component configuration please overwrite this
        method reading the data on self.conf.
        """
        pass

    def shutdown(self):
        """ If you have resources that will hang after the shutdown please
        overwrite this method and close/unload those resources.
        """
        pass


class TornadoHandler(tornado.web.RequestHandler):
    """ Base request handler to be used on a Firenado application.
    It provides session and handles component paths.
    """
    def __init__(self, application, request, **kwargs):
        self.component = None
        self.session = None
        self.__template_variables = dict()
        super(TornadoHandler, self).__init__(application, request, **kwargs)

    def initialize(self, component):
        self.component = component

    def add_variable_to_template(self, name, variable):
        """ Add a variable to a dict that will be added to the template during
        the render or render_string execution.
        """
        self.__template_variables[name] = variable

    def authenticated(self):
        """ Returns if the current user is authenticated. If the current user
        is set then we consider authenticated.

        :return: bool True is current user is set
        """
        return self.current_user is not None

    def is_mobile(self):
        from mobiledetect import MobileDetect
        if 'User-Agent' in self.request.headers:
            return MobileDetect(
                useragent=self.request.headers['User-Agent']
            ).is_mobile()
        return False

    @session.read
    def prepare(self):
        self.component.before_request(self)
        self.before_request()

    @session.write
    def on_finish(self):
        self.after_request()
        self.component.after_request(self)

    def after_request(self):
        """Called after the end of a request.

        Override this method to perform cleanup, logging, etc.
        This method is a counterpart to `prepare`.  ``on_finish`` may
        not produce any output, as it is called after the response
        has been sent to the client.

        Use this method instead of `on_finish` to avoid the session to break
        and use session features. This method will be called by `on_finish`
        with a valid session.

        This method is called before it's component's after_request if defined.
        """
        pass

    def before_request(self):
        """Called at the beginning of a request before  `get`/`post`/etc.

        Override this method to perform common initialization regardless
        of the request method.

        Use this method instead of `prepare` to avoid the session to break and
        use session features. This method will be called by `prepare` with
        a valid session.

        This method is called after it's component's before_request if defined.

        Asynchronous support: Use ``async def`` or decorate this method with
        `.gen.coroutine` to make it asynchronous.
        If this method returns an  ``Awaitable`` execution will not proceed
        until the ``Awaitable`` is done.

        .. versionadded:: 0.1.10
           Asynchronous support.
        """
        pass

    def render_string(self, template_name, **kwargs):
        ignore_component = False
        application_component = None
        for key in ('ignore_component', 'component',):
            if key in kwargs:
                if key is 'ignore_component':
                    ignore_component = kwargs[key]
                if key is 'component':
                    pass
        kwargs['user_agent'] = self.user_agent if hasattr(
            self, 'user_agent') else None
        kwargs['credential'] = self.credential if hasattr(
            self, 'credential') else None
        for name, variable in iteritems(self.__template_variables):
            kwargs[name] = variable
        if self.ui:
            return super(TornadoHandler, self).render_string(
                template_name, **kwargs)
        else:
            # TODO: After a redirect I'm still hitting here.
            # Need to figure out what is going on.
            self._finished = False
            return None

    def write_error(self, status_code, **kwargs):
        error_stack = {'code': status_code}

        exc_info = None
        for key in kwargs:
            if key == 'exc_info':
                exc_info = kwargs[key]
        error = exc_info[1]

        if type(error) == JSONError:
            error_stack.update(error.data)
            response = dict(data=None, error=error_stack)
            self.write(response)
        else:
            raise error

    def get_data_connected(self):
        return self.application

    def get_firenado_template_path(self):
        """Override to customize the firenado template path for each handler.

        By default, we use the ``firenado_template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        return self.application.settings.get('firenado_template_path')

    def get_rooted_path(self, path):
        from .util.url_util import rooted_path
        root = firenado.conf.app['url_root_path']
        return rooted_path(root, path)

    def get_template_path(self):
        """Override to customize template path for each handler.

        By default, we use the ``template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        if self.component is None:
            # This is the default behaviour provided by Tornado.
            # No components on the request no fancy template path.
            return super(TornadoHandler, self).get_template_path()
        else:
            return self.component.get_template_path()

    def create_template_loader(self, template_path):
        """Returns a new template loader for the given path.

        May be overridden by subclasses.  By default returns a
        directory-based loader on the given path, using the
        ``autoescape`` application setting.  If a ``template_loader``
        application setting is supplied, uses that instead.
        """
        settings = self.application.settings
        kwargs = {}
        if 'autoescape' in settings:
            # autoescape=None means "no escaping", so we have to be sure
            # to only pass this kwarg if the user asked for it.
            kwargs['autoescape'] = settings['autoescape']
        return FirenadoComponentLoader(
            template_path, component=self.component, **kwargs)


class FirenadoComponentLoader(Loader):
    """ A template loader that loads from a single root directory.
    """
    def __init__(self, root_directory, component=None, **kwargs):
        # TODO: Check if we should alter/use the root_directory value
        # here or on the resolve_path method.
        self.component = component
        super(FirenadoComponentLoader, self).__init__(root_directory, **kwargs)

    def resolve_path(self, name, parent_path=None):
        """ When a template name comes with a ':' it means a template from
        another component is being referenced. The component template will be
        resolved and passed to the original Tornado resolve_path method.

        :param name: The template name
        :param parent_path: The template parent path
        :return: Tornado resolve_path result.
        """
        logger.debug("Resolving template %s." % name)
        name_resolved = name
        if ':' in name:
            nameX = name.split(':')
            component_name = nameX[0]
            name_resolved = os.path.join(
                self.component.application.components[
                    component_name].get_template_path(), nameX[-1])
        if name != name_resolved:
            logger.debug("Template %s resolved at %s." % (name, name_resolved))

        return super(FirenadoComponentLoader,
                     self).resolve_path(name_resolved, parent_path)


# TODO: We need to create a class to avoid those methods repeated here.
class TornadoWebSocketHandler(tornado.websocket.WebSocketHandler):

    def __init__(self, application, request, **kwargs):
        self.component = None
        self.__template_variables = dict()
        super(TornadoWebSocketHandler, self).__init__(application,
                                                      request, **kwargs)

    def initialize(self, component):
        self.component = component

    def add_variable_to_template(self, name, variable):
        """ Add a variable to a dict that will be added to the template during
        the render or render_string execution.
        """
        self.__template_variables[name] = variable

    @session.read
    def prepare(self):
        self.component.before_handler(self)

    @session.write
    def on_finish(self):
        self.component.after_handler(self)

    def render_string(self, template_name, **kwargs):
        ignore_component = False
        application_component = None
        for key in ('ignore_component', 'component',):
            if key in kwargs:
                if key is 'ignore_component':
                    ignore_component = kwargs[key]
                if key is 'component':
                    pass
        kwargs['user_agent'] = self.user_agent if hasattr(
            self, 'user_agent') else None
        kwargs['credential'] = self.credential if hasattr(
            self, 'credential') else None
        for name, variable in iteritems(self.__template_variables):
            kwargs[name] = variable
        if self.ui:
            return super(TornadoWebSocketHandler, self).render_string(
                template_name, **kwargs)
        else:
            # TODO: After a redirect I'm still hitting here.
            # Need to figure out what is going on.
            self._finished = False
            return None

    def write_error(self, status_code, **kwargs):
        error_stack = {'code': status_code}

        exc_info = None
        for key in kwargs:
            if key == 'exc_info':
                exc_info = kwargs[key]
        error = exc_info[1]

        if type(error) == JSONError:
            error_stack.update(error.data)
            response = dict(data=None, error=error_stack)
            self.write(response)
        else:
            raise error

    def get_firenado_template_path(self):
        """Override to customize the firenado template path for each handler.

        By default, we use the ``firenado_template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        return self.application.settings.get('firenado_template_path')

    def get_template_path(self):
        """Override to customize template path for each handler.

        By default, we use the ``template_path`` application setting.
        Return None to load templates relative to the calling file.
        """
        if self.component is None:
            # This is the default behaviour provided by Tornado.
            # No components on the request no fancy template path.
            return super(TornadoWebSocketHandler, self).get_template_path()
        else:
            return self.component.get_template_path()

    def create_template_loader(self, template_path):
        """Returns a new template loader for the given path.

        May be overridden by subclasses.  By default returns a
        directory-based loader on the given path, using the
        ``autoescape`` application setting.  If a ``template_loader``
        application setting is supplied, uses that instead.
        """
        settings = self.application.settings
        kwargs = {}
        if 'autoescape' in settings:
            # autoescape=None means "no escaping", so we have to be sure
            # to only pass this kwarg if the user asked for it.
            kwargs['autoescape'] = settings['autoescape']
        return FirenadoComponentLoader(
            template_path, component=self.component, **kwargs)


#TODO: This is iFlux migration leftover. Is that necessary?.
class JSONError(tornado.web.HTTPError):

    def __init__(self, status_code, log_message=None, *args, **kwargs):
        data = {}
        self.data.update(log_message)
        if not isinstance(log_message, string_types):
            json_log_message = self.data
            json_log_message['code'] = status_code
            json_log_message = json_encode(json_log_message)
        super(JSONError, self).__init__(
            status_code, json_log_message, *args, **kwargs)



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cartola import sysexits
import firenado.conf
import logging
import os
import six
from six import iteritems
import sys
from tornado import gen

if six.PY3:
    try:
        import importlib
        reload = importlib.reload
    except AttributeError:
        # PY33
        import imp
        reload = imp.reload

logger = logging.getLogger(__name__)


class FirenadoLauncher(object):

    def __init__(self, **settings):
        self.app = settings.get("app", None)
        self.path = settings.get("path", None)
        if self.path is not None:
            sys.path.append(self.path)
        if self.app:
            os.environ["CURRENT_APP"] = self.app
        if os.environ.get("CURRENT_APP"):
            self.app = os.environ.get("CURRENT_APP")
        self.addresses = settings.get("addresses", None)
        self.dir = settings.get("dir", None)
        self.port = settings.get("port", None)
        self.socket = settings.get("socket", None)

        # Fixing and cleaning PYTHONPATH and sys.path
        # This is useful so we can run a process launcher with the same
        # PYTHONPATH from the parent process
        real_pythonpaths = sys.path[:]
        if "PYTHONPATH" in os.environ and os.environ['PYTHONPATH'] is not None:
            current_pythonpaths = os.environ['PYTHONPATH'].split(":")
            for path in current_pythonpaths:
                if path.strip() != "":
                    if path.strip() not in real_pythonpaths:
                        real_pythonpaths.append(path.strip())
            for path in sys.path:
                if path.strip() != "":
                    if path.strip() not in real_pythonpaths:
                        real_pythonpaths.append(path.strip())
        sys.path = real_pythonpaths
        os.environ['PYTHONPATH'] = ":".join(real_pythonpaths)

    def load(self):
        return None

    def launch(self):
        return None


class ProcessLauncher(FirenadoLauncher):
    try:
        import pexpect
        process = None  # type: pexpect.spawn
    except ImportError:
        logger.debug("The pexpect module ins't isn't installed. Consider "
                     "installing it in order to launch applications using "
                     "ProcessLauncher.")

    def __init__(self, **settings):
        super(ProcessLauncher, self).__init__(**settings)
        self.process = None
        self.process_callback = None
        self.logfile = settings.get("logfile", None)
        self.command = None
        self.response = None

    def load(self):
        sys.base_exec_prefix
        firenado_script = os.path.join(firenado.conf.ROOT, "bin",
                                       "firenado-cli.py")
        self.command = "%s %s app run" % (sys.executable, firenado_script)
        if self.dir is not None:
            dir_parameter = "--dir=%s" % self.dir
            self.command = "%s %s" % (self.command, dir_parameter)

        if self.socket is None:
            if self.addresses is not None:
                addresses_parameter = "--port=%s" % self.addresses
                self.command = "%s %s" % (self.command, addresses_parameter)
            if self.port is not None:
                port_parameter = "--port=%s" % self.port
                self.command = "%s %s" % (self.command, port_parameter)
        else:
            socket_parameter = "--port=%s" % self.socket
            self.command = "%s %s" % (self.command, socket_parameter)

    @gen.coroutine
    def read_process(self):
        import pexpect
        self.process_callback.stop()
        try:
            # Simple way to catch everything is wait for a new line:
            #
            yield self.process.expect("\n", async_=True)
        except pexpect.TIMEOUT:
            logger.warning("Reached timeout")
            pass
        self.process_callback.start()

    @gen.coroutine
    def launch(self):
        import pexpect
        import tornado.ioloop
        logger.info("Launching %s" % self.command)
        parameters = {
            'command': self.command,
            'encoding': "utf-8"
        }
        if self.logfile is not None:
            parameters['logfile'] = self.logfile
        self.process = pexpect.spawn(**parameters)
        yield self.process.expect(
            [r"[Firenado server started successfully].*"], async_=True)
        self.process_callback = tornado.ioloop.PeriodicCallback(
            self.read_process,
            400
        )
        self.process_callback.start()

    def send(self, line):
        logger.info("Sending line {}".format(line))
        self.process.sendline(line)

    def shutdown(self):
        logger.warning("Shutting down process launcher.")
        self.process.terminate(force=True)

    def is_alive(self):
        return self.process.isalive()


class TornadoLauncher(FirenadoLauncher):

    def __init__(self, **settings):
        super(TornadoLauncher, self).__init__(**settings)
        self.http_server = None
        self.application = None
        if self.dir is not None:
            os.chdir(self.dir)
        if self.app is not None or self.dir is not None:
            reload(firenado.conf)
        self.MAX_WAIT_SECONDS_BEFORE_SHUTDOWN = firenado.conf.app[
            'wait_before_shutdown']
        if self.addresses is None or self.addresses == ['']:
            if firenado.conf.app['addresses']:
                self.addresses = firenado.conf.app['addresses']
            else:
                self.addresses = firenado.conf.app['default_addresses']
        if self.port is None:
            self.port = firenado.conf.app['port']

    def load(self):
        from .tornadoweb import TornadoApplication
        # TODO: Resolve module if doesn't exists
        if firenado.conf.app['pythonpath']:
            sys.path.append(firenado.conf.app['pythonpath'])
        self.application = TornadoApplication(**firenado.conf.app['settings'])

    def launch(self):
        import signal
        import tornado.httpserver
        signal.signal(signal.SIGTERM, self.sig_handler)
        signal.signal(signal.SIGINT, self.sig_handler)
        if os.name == "posix":
            signal.signal(signal.SIGTSTP, self.sig_handler)
        self.http_server = tornado.httpserver.HTTPServer(self.application)
        listening_count = 0
        listening_what = "socket"
        if firenado.conf.app['socket'] or self.socket:
            from tornado.netutil import bind_unix_socket
            socket_path = firenado.conf.app['socket']
            if self.socket:
                socket_path = self.socket
            socket = bind_unix_socket(socket_path)
            self.http_server.add_socket(socket)
            logger.info("Firenado listening at socket ""%s" %
                        socket.getsockname())
            listening_count += 1
        else:
            listening_what = "addresses:port"
            if len(self.addresses):
                for address in self.addresses:
                    if self.add_sockets(self.port, address):
                        listening_count += 1
            else:
                if self.add_sockets(self.port):
                    listening_count += 1
        if listening_count:
            if listening_count > 1:
                listening_what = "%ss" % listening_what
            logger.info("Firenado server started successfully. Listening at %s"
                        " %s." % (listening_count, listening_what))
            if firenado.conf.app['process']['num_processes'] is not None:
                import tornado.process
                num_processes = firenado.conf.app['process']['num_processes']
                max_restarts = firenado.conf.app['process']['max_restarts']
                num_processes_alert = num_processes
                if num_processes == 0:
                    num_processes_alert = ("0 (assuming %s as cpu count)" %
                                           tornado.process.cpu_count())
                logger.info("Tornado set to start %s processes with %s max "
                            "restarts." % (num_processes_alert, max_restarts))
                tornado.process.fork_processes(num_processes, max_restarts)
            tornado.ioloop.IOLoop.current().start()
        else:
            logger.critical("Firenado unable to start.")
            sysexits.exit_fatal(sysexits.EX_SOFTWARE)

    def sig_handler(self, sig, _):
        """ Handle the signal sent to the process
        :param sig:  Signal set to the process
        :param _: Frame is not being used
        """
        import tornado.ioloop
        from tornado.process import task_id
        tid = task_id()
        pid = os.getpid()
        if tid is None:
            logger.warning("main process (pid %s) caught signal: %s" %
                           (pid, sig))
        else:
            logger.warning("child %s (pid %s) caught signal: %s" %
                           (tid, pid, sig))
        tornado.ioloop.IOLoop.current().add_callback_from_signal(self.shutdown)

    def add_sockets(self, port, address=None):
        from socket import gaierror
        try:
            from tornado.netutil import bind_sockets
            if address:
                sockets = bind_sockets(port, address.strip())
            else:
                sockets = bind_sockets(port)
                address = "127.0.0.1"
            self.http_server.add_sockets(sockets)
            logger.info("Firenado listening at http://%s:%s." %
                        (address.strip(), port))
            return True
        except gaierror as error:
            logger.error("Firenado failed to listen at http://%s:%s."
                         % (address.strip(), port))
        except OSError as error:
            logger.error("Firenado failed to listen at http://%s:%s. "
                         "Check if another process is listening at "
                         "this port or if you provided a dns name and"
                         "the correspondent ip in the addresses"
                         "configuration or if the machine owns the ip "
                         "Fireando will start to listen." %
                         (address, port))
        return False

    def shutdown(self):
        import time
        import tornado.ioloop
        from tornado.process import task_id
        tid = task_id()
        pid = os.getpid()
        if tid is None:
            logger.info("main process (pid %s): stopping http server" % pid)
        else:
            logger.info("child %s (pid %s): stopping http server" % (tid, pid))
        for key, component in iteritems(self.application.components):
            component.shutdown()
        self.http_server.stop()

        io_loop = tornado.ioloop.IOLoop.current()

        if self.MAX_WAIT_SECONDS_BEFORE_SHUTDOWN == 0:
            io_loop.stop()
            if tid is None:
                logger.info("main process (pid %s): application is down" % pid)
            else:
                logger.info("child %s (pid %s): application is down" %
                            (tid, pid))
        else:
            if tid is None:
                logger.info("main process (pid %s): shutdown in %s seconds "
                            "..." %
                            (pid, self.MAX_WAIT_SECONDS_BEFORE_SHUTDOWN))
            else:
                logger.info("child %s (pid %s): shutdown in %s seconds ..." %
                            (tid, pid, self.MAX_WAIT_SECONDS_BEFORE_SHUTDOWN))
            deadline = time.time() + self.MAX_WAIT_SECONDS_BEFORE_SHUTDOWN

            def stop_loop():
                now = time.time()
                if now < deadline:
                    io_loop.add_timeout(now + 1, stop_loop)
                else:
                    io_loop.stop()
                    if tid is None:
                        logger.info("main process (pid %s): application is "
                                    "down" % pid)
                    else:
                        logger.info("child %s (pid %s): application is down" %
                                    (tid, pid))
            stop_loop()



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cartola import sysexits
import yaml
import logging
import os


def get_app_defaults():
    """ Return the application configuration defaults
    :return:
    """
    app = dict()
    app['default_addresses'] = ["::", "0.0.0.0"]
    app['addresses'] = app['default_addresses']
    app['component'] = None
    # TODO: Are we using current_user_key?
    app['current_user_key'] = "__FIRENADO_CURRENT_USER_KEY__"
    app['data'] = {}
    app['data']['sources'] = []
    app['id'] = None
    app['pythonpath'] = None
    app['port'] = 8888
    app['process'] = {
        'num_processes': None,
        'max_restarts': 100
    }
    app['login'] = {}
    app['login']['urls'] = {}
    app['login']['urls']['default'] = "/login"
    app['is_on_dir'] = False
    app['session'] = {
        'id_generator': "default",
    }
    app['settings'] = {}
    app['socket'] = None
    app['static_path'] = None
    app['static_url_prefix'] = "/static"
    app['type'] = "tornado"
    app['types'] = {}
    app['types']['tornado'] = {}
    app['types']['tornado']['name'] = "tornado"
    app['types']['tornado']['launcher'] = {}
    app['types']['tornado']['launcher']['class'] = "TornadoLauncher"
    app['types']['tornado']['launcher']['module'] = "firenado.launcher"
    app['url_root_path'] = None
    # Wait before shutdown is on seconds
    app['wait_before_shutdown'] = 0
    return app


def log_level_from_string(str_level):
    """ Returns the proper log level core based on a given string

    :param str_level: Log level string
    :return: The log level code
    """
    levels = {
        'CRITICAL': logging.CRITICAL,
        'ERROR': logging.ERROR,
        'WARNING': logging.WARNING,
        'INFO': logging.INFO,
        'DEBUG': logging.DEBUG,
        'NOTSET': logging.NOTSET,
    }
    try:
        return levels[str_level.upper()]
    except KeyError:
        pass
    except AttributeError:
        if str_level in [logging.DEBUG, logging.INFO, logging.WARNING,
                         logging.ERROR, logging.CRITICAL]:
            return str_level
    return logging.NOTSET


def get_config_from_package(package):
    """ Breaks a package string in module and class.

    :param package: A package string.
    :return: A config dict with class and module.
    """
    package_x = package.split('.')
    package_conf = {}
    package_conf['class'] = package_x[-1]
    package_conf['module'] = '.'.join(package_x[:-1][:])
    return package_conf


def get_class_from_name(name):
    """ Return a class reference from the class name provided as a parameter.
    Class name must be the full class reference, in another words, the module
    with the class absolute reference.

    Example:
    >>> get_class_from_name("my.module.Myclass")

    :param basestring name: Class absolute reference.
    :return: The class resolved from the absolute reference name provided.
    """
    return get_class_from_module(
        ".".join(name.split(".")[:-1]),
        name.split(".")[-1]
    )


def get_class_from_module(module, class_name):
    """ Returns a class from a module and a class name parameters.
    This function is used by get_class_from_config and get_class_from_name.

    Example:
    >>> get_class_from_module("my.module", "MyClass")

    :param basestring module: The module name.
    :param basestring class_name: The class name.
    :return: The class resolved by the module and class name provided.
    """
    import importlib
    module = importlib.import_module(module)
    return getattr(module, class_name)


def get_class_from_config(config, index="class"):
    """ Return a class from a config dict bit containing the indexes module and
    class.

    Examples:

    When class name index into the config is class:
    >>> config = {'module': "my.module", 'class': "MyClass"}
    >>> get_class_from_config(config)

    When class name index into config is custom:
    >>> config = {'module': "my.module", 'my_class': "MyClass"}
    >>> get_class_from_config(config, index="my_class")

    :param dict config: Config containing index and module information.
    :param basestring index: Index to be used to get the class name
    :return: The class resolved at the module referred into the config.
    """
    return get_class_from_module(config['module'], config[index])


def load_yaml_config_file(path):
    """ Returns the parsed structure from a yaml config file.

    :param path: Path where the yaml file is located.
    :return: The yaml configuration represented by the yaml file.
    """
    result = None
    with open(path, 'r') as steam:
        result = yaml.safe_load(steam)
    return result


def process_config(config, config_data):
    """ Populates config with data from the configuration data dict. It handles
    components, data, log, management and session sections from the
    configuration data.

    :param config: The config reference of the object that will hold the
    configuration data from the config_data.
    :param config_data: The configuration data loaded from a configuration
    file.
    """
    if 'components' in config_data:
        process_components_config_section(config, config_data['components'])
    if 'data' in config_data:
        process_data_config_section(config, config_data['data'])
    if 'log' in config_data:
        process_log_config_section(config, config_data['log'])
    if 'management' in config_data:
        process_management_config_section(config, config_data['management'])
    if 'session' in config_data:
        process_session_config_section(config, config_data['session'])


def process_app_config(config, config_data):
    """ Populates config with data from the configuration data dict. It handles
    everything that process_config does plus application section.

    :param config: The config reference of the object that will hold the
    configuration data from the config_data.
    :param config_data: The configuration data loaded from a configuration
    file.
    """
    process_config(config, config_data)

    # If apps is on config data, this is running o multi app mode
    if 'apps' in config_data:
        config.is_multi_app = True
        process_apps_config_session(config, config_data['apps'])
        if 'app' in config_data:

            logger = logging.getLogger(__name__)
            logger.critical("Firenado is running in multi application mode. "
                            "The app section is only allowed in simple "
                            "applicaiton mode.")
            sysexits.exit_fatal(sysexits.EX_CONFIG)
    else:
        # If not the app definition is on the firenado config file
        if 'app' in config_data:
            process_app_config_section(config, config_data['app'])


# TODO: This is being used for the multi app configuration
def process_apps_config_session(config, apps_config):
    logger = logging.getLogger(__name__)

    class AppConf:
        def __init__(self):
            self.app = {}

    for app_config in apps_config:
        config.is_multi_app = True
        if 'name' in app_config:
            name = app_config['name']
            config.apps[name] = AppConf()
            config.apps[name].app = get_app_defaults()
            if 'file' in app_config:
                file = app_config['file']
                if not os.path.isabs(file):
                    file = os.path.abspath(
                        os.path.join(config.APP_CONFIG_PATH, file)
                    )
                if not os.path.exists(file):
                    logger.critical("The file %s doesn't exists. Please check"
                                    "your apps definition and inform a valid "
                                    "file." % file)
                    sysexits.exit_fatal(sysexits.EX_CONFIG)
                file_app_config = load_yaml_config_file(file)
                if "app" in file_app_config:
                    process_app_config_section(config.apps[name],
                                               file_app_config['app'])
            else:
                process_app_config_section(config.apps[name], app_config)
        else:
            logger.critical("When defining an app in the apps section the name"
                            " parameter is mandatory.")
            sysexits.exit_fatal(sysexits.EX_CONFIG)
    if config.current_app_name is None:
        for app in config.apps:
            config.current_app_name = app
            break
    if config.current_app_name in config.apps:
        config.app = config.apps[config.current_app_name].app
    else:
        logger.critical("The application %s is not defined." %
                        config.current_app_name)
        sysexits.exit_fatal(sysexits.EX_CONFIG)


def process_app_config_section(config, app_config):
    """ Processes the app section from a configuration data dict.

    :param config: The config reference of the object that will hold the
    configuration data from the config_data.
    :param app_config: App section from a config data dict.
    """
    if 'addresses' in app_config:
        config.app['addresses'] = app_config['addresses']
    if 'component' in app_config:
        config.app['component'] = app_config['component']
    if 'data' in app_config:
        if 'sources' in app_config['data']:
            config.app['data']['sources'] = app_config['data']['sources']
    if 'id' in app_config:
        config.app['id'] = app_config['id']
    if 'login' in app_config:
        if 'urls' in app_config['login']:
            for url in app_config['login']['urls']:
                config.app['login']['urls'][url['name']] = url['value']
    if 'pythonpath' in app_config:
        config.app['pythonpath'] = app_config['pythonpath']
    if 'port' in app_config:
        config.app['port'] = app_config['port']
    if 'process' in app_config:
        if 'num_processes' in app_config['process']:
            config.app['process']['num_processes'] = app_config[
                'process']['num_processes']
    if 'url_root_path' in app_config:
        root_url = app_config['url_root_path'].strip()
        if root_url[0] == "/":
            root_url = root_url[1:]
        if root_url == "":
            root_url = None
        config.app['url_root_path'] = root_url
    if 'settings' in app_config:
        config.app['settings'] = app_config['settings']
    if 'socket' in app_config:
        config.app['socket'] = app_config['socket']
    if 'static_path' in app_config:
        config.app['static_path'] = app_config['static_path']
    if 'static_url_prefix' in app_config:
        config.app['static_url_prefix'] = app_config['static_url_prefix']
    if 'type' in app_config:
        config.app['type'] = app_config['type']
    if 'types' in app_config:
        for app_type in app_config['types']:
            app_type['launcher'] = get_config_from_package(
                app_type['launcher'])
            config.app['types'][app_type['name']] = app_type
    if 'wait_before_shutdown' in app_config:
        config.app['wait_before_shutdown'] = app_config['wait_before_shutdown']


def process_components_config_section(config, components_config):
    """ Processes the components section from a configuration data dict.

    :param config: The config reference of the object that will hold the
    configuration data from the config_data.
    :param components_config: Data section from a config data dict.
    """
    for component_config in components_config:
        if 'id' not in component_config:
            raise Exception('The component %s was defined without an id.' %
                            component_config)
        component_id = component_config['id']
        if component_id not in config.components:
            config.components[component_id] = {}
            config.components[component_id]['enabled'] = False
            config.components[component_id]['config'] = {}
        if 'class' in component_config:
            class_config_x = component_config['class'].split('.')
            config.components[component_id]['class'] = class_config_x[-1]
            config.components[component_id]['module'] = '.'.join(
                class_config_x[:-1])
        if 'enabled' in component_config:
            config.components[component_id]['enabled'] = bool(
                component_config['enabled'])


def process_data_config_section(config, data_config):
    """ Processes the data configuration section from the configuration
    data dict.

    :param config: The config reference of the object that will hold the
    configuration data from the config_data.
    :param data_config: Data configuration section from a config data dict.
    """
    if 'connectors' in data_config:
        for connector in data_config['connectors']:
            config.data['connectors'][
                connector['name']] = get_config_from_package(
                connector['class'])
    if 'sources' in data_config:
        if data_config['sources']:
            for source in data_config['sources']:
                config.data['sources'][source['name']] = source
                del config.data['sources'][source['name']]['name']


def process_log_config_section(config, log_config):
    """ Processes the log section from a configuration  data dict.

    :param config: The config reference of the object that will hold the
    configuration data from the config_data.
    :param log_config: Log section from a config data dict.
    """
    if 'format' in log_config:
        config.log['format'] = log_config['format']
    if 'level' in log_config:
        config.log['level'] = log_level_from_string(log_config['level'])


def process_management_config_section(config, management_config):
    """ Processes the management section from a configuration data dict.

    :param config: The config reference of the object that will hold the
    configuration data from the config_data.
    :param management_config: Management section from a config data dict.
    """
    if 'commands' in management_config:
        for command in management_config['commands']:
            config.management['commands'].append(command)


def process_session_config_section(config, session_config):
    """ Processes the session section from the configuration data dict.

    :param config: The config reference of the object that will hold the
    configuration data from the config_data.
    :param session_config: Session configuration section from a config data
    dict.
    """
    # Setting session type as file by default
    config.session['type'] = 'file'
    if 'enabled' in session_config:
        config.session['enabled'] = session_config['enabled']
    if 'type' in session_config:
        config.session['type'] = session_config['type']
        if config.session['type'] == 'file':
            if 'path' in session_config:
                config.session['file']['path'] = session_config['path']
        if config.session['type'] == 'redis':
            if 'data' in session_config:
                if 'source' in session_config['data']:
                    config.session['redis']['data']['source'] = session_config[
                        'data']['source']
    if 'handlers' in session_config:
        for handler in session_config['handlers']:
            handler_class_x = handler['class'].split('.')
            handler['class'] = handler_class_x[-1]
            handler['module'] = '.'.join(handler_class_x[:-1][:])
            config.session['handlers'][handler['name']] = handler
            del config.session['handlers'][handler['name']]['name']
    if 'encoders' in session_config:
        for encoder in session_config['encoders']:
            encoder_class_x = encoder['class'].split('.')
            encoder['encoder'] = encoder_class_x[-1]
            encoder['class'] = encoder_class_x[-1]
            encoder['module'] = '.'.join(encoder_class_x[:-1][:])
            config.session['encoders'][encoder['name']] = encoder
            del config.session['encoders'][encoder['name']]['name']
    if 'id_generators' in session_config:
        for generator in session_config['id_generators']:
            generator_ref_x = generator['function'].split('.')
            generator['function'] = generator_ref_x[-1]
            generator['module'] = '.'.join(generator_ref_x[:-1][:])
            config.session['id_generators'][generator['name']] = generator
            del config.session['id_generators'][generator['name']]['name']
    if 'name' in session_config:
        config.session['name'] = session_config['name']
    if 'life_time' in session_config:
        config.session['life_time'] = session_config['life_time']
    if 'callback_hiccup' in session_config:
        config.session['callback_hiccup'] = session_config['callback_hiccup']
    if 'callback_time' in session_config:
        config.session['callback_time'] = session_config['callback_time']
    if 'prefix' in session_config:
        config.session['prefix'] = session_config['prefix']
    if 'purge_limit' in session_config:
        config.session['purge_limit'] = session_config['purge_limit']
    if 'encoder' in session_config:
        if session_config['encoder'] in config.session['encoders']:
            config.session['encoder'] = session_config['encoder']
        else:
            logger = logging.getLogger(__name__)
            logger.critical("The session encoder \"{}\" is not defined."
                            "".format(session_config['encoder']))
            sysexits.exit_fatal(sysexits.EX_CONFIG)



# -*- coding: UTF-8 -*-
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from behave import fixture, use_fixture
from firenado import launcher
from unittest.case import TestCase


@fixture
def firenado_app(context, timeout=30, **kwargs):
    context.launcher = launcher.FirenadoLauncher()
    yield context.launcher


@fixture
def tester(context, timeout=1, **kwargs):
    context.tester = TestCase()
    yield context.tester


def before_all(context):
    use_fixture(tester, context)



#!/usr/bin/env python
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

from firenado.util.sqlalchemy_util import Base, base_to_dict
from sqlalchemy import Column, String
from sqlalchemy.types import Integer, DateTime
from sqlalchemy.sql import text
import unittest


class TestBase(Base):

    __tablename__ = "test"

    id = Column("id", Integer, primary_key=True)
    username = Column("username", String(150), nullable=False)
    first_name = Column("first_name", String(150), nullable=False)
    last_name = Column("last_name", String(150), nullable=False)
    password = Column("password", String(150), nullable=False)
    email = Column("email", String(150), nullable=False)
    created = Column("created", DateTime, nullable=False,
                     server_default=text("now()"))
    modified = Column("modified", DateTime, nullable=False,
                      server_default=text("now()"))


class BaseToDictTestCase(unittest.TestCase):

    def setUp(self):
        self.test_object = TestBase()
        self.test_object.id = 1
        self.test_object.username = "anusername"
        self.test_object.password = "apassword"
        self.test_object.first_name = "Test"
        self.test_object.last_name = "Object"
        self.test_object.email = "test@example.com"

    def test_base_to_dict(self):
        dict_from_base = base_to_dict(self.test_object)
        self.assertEqual(dict_from_base['id'], self.test_object.id)
        self.assertEqual(dict_from_base['username'], self.test_object.username)
        self.assertEqual(dict_from_base['password'], self.test_object.password)
        self.assertEqual(dict_from_base['first_name'],
                         self.test_object.first_name)
        self.assertEqual(dict_from_base['last_name'],
                         self.test_object.last_name)
        self.assertEqual(dict_from_base['email'], self.test_object.email)
        self.assertEqual(dict_from_base['created'], self.test_object.created)
        self.assertEqual(dict_from_base['modified'], self.test_object.modified)

    def test_base_to_dict(self):
        dict_from_base = base_to_dict(self.test_object,
                                      ["id", "username", "first_name"])
        self.assertEqual(dict_from_base['id'], self.test_object.id)
        self.assertEqual(dict_from_base['username'], self.test_object.username)
        self.assertEqual(dict_from_base['first_name'],
                         self.test_object.first_name)
        self.assertTrue('password' not in dict_from_base)
        self.assertTrue('last_name' not in dict_from_base)
        self.assertTrue('email' not in dict_from_base)
        self.assertTrue('created' not in dict_from_base)
        self.assertTrue('modified' not in dict_from_base)



#!/usr/bin/env python
#
# Copyright 2015-2019 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

from firenado import security
import unittest


class MockRequest:
    """ Mock the request being from the mock handler.
    """

    def __init__(self):
        self.headers = {}


class MockHandler:
    """ Mock the handler being decorated by the security functions.
    """
    def __init__(self):
        self.status = 200
        self.response = None
        self.request = MockRequest()

    def set_status(self, status):
        self.status = status

    def write(self, value):
        self.response = value

    @security.only_xhr
    def get_only_xhr(self):
        pass


class SecurityTestCase(unittest.TestCase):

    def setUp(self):
        """ Setting up a mock handler to test security decorators.
        """
        self.handler = MockHandler()

    def test_only_xhr(self):
        """ Check if the xhr will allow the request to go when XMLHttpRequest
        is found in the headers and if a 403 will be thrown when not found.
        """
        self.handler.request.headers['X-Requested-With'] = "XMLHttpRequest"
        self.handler.get_only_xhr()
        self.assertEqual(self.handler.status, 200)
        self.assertEqual(self.handler.response, None)

        self.handler.request.headers.pop('X-Requested-With')
        self.handler.get_only_xhr()
        self.assertEqual(self.handler.status, 403)
        self.assertEqual(self.handler.response, "This is a XMLHttpRequest "
                                                "request only.")



#!/usr/bin/env python
#
# Copyright 2015-2018 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import firenado.tornadoweb
import logging
import tornado.web
import os

logger = logging.getLogger(__name__)


class StaticMapsComponent(firenado.tornadoweb.TornadoComponent):
    """ The static maps component helps to map static handlers using a config
    file.
    """

    def __init__(self, name, application):
        firenado.tornadoweb.TornadoComponent.__init__(self, name, application)
        self.static_root = None
        self.static_maps = {}

    def get_handlers(self):
        """ Returns the handlers defined on the static_maps.yml file located
        at the app config directory.

        Returns: An array of static handlers to be added to the app.
        """
        handlers = []
        self.static_root = self.application.get_app_component(
        ).get_component_path()
        if self.conf:
            if 'maps' in self.conf:
                if self.conf['maps'] is None:
                    logger.warning("Maps configuration is empty. Finish the"
                                   "static maps configuration.")
                    return handlers
                for map_item in self.conf['maps']:
                    logger.debug("Mapping %s handlers." % map_item['name'])
                    self.static_maps[map_item['name']] = {}
                    self.static_maps[
                        map_item['name']]['root'] = self.static_root
                    if 'root' in map_item:
                        if os.path.isabs(map_item['root']):
                            self.static_maps[
                                map_item['name']]['root'] = map_item['root']
                        else:
                            self.static_maps[
                                map_item['name']]['root'] = os.path.abspath(
                                os.path.join(self.static_root,
                                             map_item['root']))
                    if 'handlers' in map_item:
                        if map_item['handlers'] is None:
                            logger.warning("There is no handles mapped in the"
                                           " static maps config file.")
                        else:
                            handlers = handlers + self.get_static_handlers(
                                map_item)
        else:
            logger.warning("No static maps configurations were provided.")
        return handlers

    def get_static_handlers(self, map_item):
        static_handlers = []
        for handler in map_item['handlers']:
            handler_path = ""
            if 'path' in handler:
                handler_path = handler['path']
            if 'name_on_path' in handler:
                if handler['name_on_path']:
                    handler_path = os.path.join(handler['name'], handler_path)
            if 'map_on_path' in handler:
                if handler['map_on_path']:
                    handler_path = os.path.join(map_item['name'], handler_path)
            logger.debug(
                "Mapping handler %s on directory %s on the map %s." % (
                    handler['handler'], handler_path, map_item['name']))
            static_handler = (
                r"%s" % handler['handler'],
                tornado.web.StaticFileHandler,
                {"path": os.path.join(
                    self.static_maps[map_item['name']]['root'],
                    handler_path)})
            static_handlers.append(static_handler)
        return static_handlers

    def get_config_file(self):
        return "static_maps"



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2015-2019 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division,
                        print_function, with_statement)

import argparse
import firenado.conf
from firenado.util.argparse_util import FirenadoArgumentError
from firenado.util.argparse_util import FirenadoArgumentParser
import logging
import os
import sys
from six import iteritems
from tornado import template

logger = logging.getLogger(__name__)

# Commands will be registered here. This is done by ManagementCommand
command_categories = dict()


def run_from_command_line():
    """ Run Firenado's management commands from a command line
    """
    for commands_conf in firenado.conf.management['commands']:
        logger.debug("Loading %s commands from %s." % (
            commands_conf['name'],
            commands_conf['module']
        ))
        exec('import %s' % commands_conf['module'])
    command_index = 1
    for arg in sys.argv[1:]:
        command_index += 1
        if arg[0] != "-":
            break
    parser = FirenadoArgumentParser(prog=os.path.split(sys.argv[0])[1],
                                    add_help=False)
    parser.add_argument("-h", "--help", default=argparse.SUPPRESS)
    parser.add_argument("command", default="help", help="Command to executed")

    try:
        namespace = parser.parse_args(sys.argv[1:command_index])
        if not command_exists(namespace.command):
            show_command_line_usage(parser)
        else:
            run_command(namespace.command, sys.argv[command_index-1:])
    except FirenadoArgumentError:
        show_command_line_usage(parser, True)


def get_command_header(parser, usage_message="", usage=False):
    """ Return the command line header

    :param parser:
    :param usage_message:
    :param usage:
    :return: The command header
    """
    loader = template.Loader(os.path.join(
        firenado.conf.ROOT, 'management', 'templates', 'help'))
    return loader.load("header.txt").generate(
        parser=parser, usage_message=usage_message, usage=usage,
        firenado_version=".".join(map(str, firenado.__version__))).decode(
        sys.stdout.encoding)


def show_command_line_usage(parser, usage=False):
    """ Show the command line help
    """
    help_header_message = get_command_header(parser, "command", usage)
    loader = template.Loader(os.path.join(
        firenado.conf.ROOT, 'management', 'templates', 'help'))
    command_template = "  {0.name:15}{0.description:40}"
    help_message = loader.load("main_command_help.txt").generate(
        command_categories=command_categories,
        command_template=command_template
    ).decode(sys.stdout.encoding)
    # TODO: This print has to go. Use proper stream instead(stdout or stderr)
    print(''.join([help_header_message, help_message]))


def command_exists(command):
    """ Check if the given command was registered. In another words if it
    exists.
    """
    for category, commands in iteritems(command_categories):
        for existing_command in commands:
            if existing_command.match(command):
                return True
    return False


def run_command(command, args):
    """ Run all tasks registered in a command.
    """
    for category, commands in iteritems(command_categories):
        for existing_command in commands:
            if existing_command.match(command):
                existing_command.run(args)


class ManagementCommand(object):
    """ Defines a management command. Commands are classified by categories and
    the Firenado category is the default one. Those commands are shipped with
    the framework.
    Developers can create new categories of commands and commands and
    distribute them with their application and/or components.
    """

    def __init__(self, name, description, cmd_help, category=None,
                 sub_commands=None, tasks=None, parent=None):
        """ To register a management command it is necessary inform the
        category you the command belongs, it's name and description and a
        meaningful help to be displayed.

        :param name: Command's name
        :param description: Command's description
        :param cmd_help: Meaningful help to be displayed
        :param category: The category the command belongs to
        :param sub_commands: Sub commands aggregated into the command
        :param tasks: Tasks to be executed when this command is called
        """
        self.category = category
        self.name = name
        self.commands = self.name.split("(")
        if len(self.commands) > 1:
            self.commands[1] = "%s%s" % (self.commands[0],
                                         self.commands[1][:-1])
        self.description = description
        self.help = cmd_help
        self.sub_commands = sub_commands
        self.tasks = []
        self.parent = None
        if tasks is not None:
            if isinstance(tasks, list):
                for task in tasks:
                    self.tasks.append(task(self))
            else:
                self.tasks.append(tasks(self))
        if category is not None:
            if category not in command_categories:
                command_categories[category] = []
            command_categories[category].append(self)

    def get_help(self):
        return self.help

    def match(self, command):
        return command in self.commands

    def run(self, args):
        has_sub_commands = False
        subcommands_resolved = False
        if self.sub_commands:
            has_sub_commands = True
            # TODO handle args with size 1
            unresolved_args = args[1:]
            if len(unresolved_args):
                for subcommand in self.sub_commands:
                    if subcommand.name == unresolved_args[0]:
                        subcommand.run(unresolved_args)
                        subcommands_resolved = True
                        break
        if not has_sub_commands:
            self.run_tasks(args)
            exit(0)
        else:
            if not subcommands_resolved:
                from tornado.template import Template
                msg_help = self.get_help()
                parser = FirenadoArgumentParser(
                        prog=os.path.split(
                            sys.argv[0])[1], usage='%(prog)s [options]',)
                if isinstance(msg_help, Template):
                    msg_help = msg_help.generate(parser=parser)
                print(get_command_header(parser, msg_help, True))
            exit(0)

    def run_tasks(self, args):
        cmd_parser = FirenadoArgumentParser(
            prog=os.path.split(sys.argv[0])[1], usage='%(prog)s [options]',)
        cmd_parser.add_argument("command", help="Command to executed")
        try:
            for task in self.tasks:
                task.add_arguments(cmd_parser)
            namespace = cmd_parser.parse_args(args)
            for task in self.tasks:
                task.run(namespace)
        except FirenadoArgumentError as error:
            command_help = ""
            for task in self.tasks:
                error_message = task.get_error_message(cmd_parser, error)
                if error_message:
                    command_help += "\n".join([command_help, error_message])
            print(command_help)


class ManagementTask(object):
    """
    Defines a management tasks. Tasks are the concrete actions executed by a
    command.
    """
    def __init__(self, action):
        self.action = action

    def add_arguments(self, parser):
        """
        Implement this method to add arguments to the current argparse parser
        being handled by the command.
        """
        pass

    def get_help(self):
        """
        Implement this method to add a help text to the help message to be
        displayed by the command.
        """
        return None

    def get_error_message(self, parser, error):
        if hasattr(error, "message"):
            return error.message
        return str(error)

    def run(self, namespace=None):
        """
        Task implementation is done here.
        """
        pass



#!/usr/bin/env python
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cartola.pagination import Paginator
import tornado.web


class Paginate(tornado.web.UIModule):

    def render(self, row_count, current_page, rows_per_page=None,
               pages_per_block=None, argument="p",
               template="toolbox:uimodules/pagination.html"):
        component_conf = self.handler.component.conf
        if 'pagination' in self.handler.component.conf:
            if 'rows_per_page' in component_conf['pagination']:
                rows_per_page = component_conf['pagination']['rows_per_page']
            if 'pages_per_block' in component_conf['pagination']:
                pages_per_block = component_conf['pagination'][
                    'pages_per_block']
        if rows_per_page is None:
            rows_per_page = 10
        if pages_per_block is None:
            pages_per_block = 10
        paginator = Paginator(row_count, current_page, rows_per_page,
                              pages_per_block)
        return self.render_string(template, argument=argument,
                                  paginator=paginator)



#!/usr/bin/env python
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import firenado
from setuptools import setup, find_packages
import sys

try:
    # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:
    # for pip <= 9.0.3
    print("error: Upgrade to a pip version newer than 10. Run \"pip install "
          "--upgrade pip\".")
    sys.exit(1)


# Solution from http://bit.ly/29Yl8VN
def resolve_requires(requirements_file):
    try:
        requirements = parse_requirements("./%s" % requirements_file,
                                          session=False)
        return [str(ir.req) for ir in requirements]
    except AttributeError:
        # for pip >= 20.1.x
        # Need to run again as the first run was ruined by the exception
        requirements = parse_requirements("./%s" % requirements_file,
                                          session=False)
        # pr stands for parsed_requirement
        return [str(pr.requirement) for pr in requirements]


def use_right_tornado(requirements):
    tornado_req = "requirements/tornado_new.txt"
    if sys.version[0] == "2":
        tornado_req = "requirements/tornado_legacy.txt"
    requirements.append(resolve_requires(tornado_req)[0])
    return requirements


with open("README.md", "r") as fh:
    long_description = fh.read()

# We still running: python setup.py sdist upload --repository=testpypi
# Twine isn't handling long_descriptions as per:
# https://github.com/pypa/twine/issues/262
setup(
    name="Firenado",
    version=firenado.get_version(),
    description="Firenado is a python web framework based on Tornado web "
                "framework/server.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license=firenado.__licence__,
    author=firenado.get_author(),
    author_email=firenado.get_author_email(),
    maintainer=firenado.get_author(),
    maintainer_email=firenado.get_author_email(),
    install_requires=use_right_tornado(resolve_requires(
        "requirements/basic.txt"
    )),
    extras_require={
        'all': resolve_requires("requirements/all.txt"),
        'redis': resolve_requires("requirements/redis.txt"),
        'pexpect': resolve_requires("requirements/pexpect.txt"),
        'schedule': resolve_requires("requirements/schedule.txt"),
        'sqlalchemy': resolve_requires("requirements/sqlalchemy.txt"),
    },
    url="https://github.com/candango/firenado",
    packages=find_packages(),
    package_dir={'firenado': "firenado"},
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: Web Environment",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    scripts=["firenado/bin/firenado-cli.py"],
    entry_points={'console_scripts': [
        "firenado = firenado.management:run_from_command_line",
    ]},
)



#!/usr/bin/env python
#
# Copyright 2015-2020 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import (absolute_import, division, print_function,
                        with_statement)

from cartola import fs
import unittest
import firenado.conf
from tests import chdir_app
import logging
import os
import six

if six.PY3:
    try:
        import importlib
        reload = importlib.reload
    except AttributeError:
        # PY33
        import imp
        reload = imp.reload


class ApplicationComponentTestCase(unittest.TestCase):
    """ Case that tests an Firenado application after being loaded from its
    configuration file.
    """

    def test_conf_root(self):
        """ Test if Firenado root matches the upper directory relative to the
        current one. """
        import os
        current_path = os.path.dirname(os.path.realpath(__file__))
        firenado_root = ("%s" % os.sep).join(current_path.split(os.sep)[:-1])
        self.assertEqual(firenado_root, firenado.conf.ROOT)

    def test_conf_root(self):
        """ Test if Firenado root matches the upper directory relative to the
        current one. """
        current_path = os.path.dirname(os.path.realpath(__file__))
        firenado_root = ("%s" % os.sep).join(current_path.split(os.sep)[:-1])
        firenado_root = os.path.join(firenado_root, "firenado")
        self.assertEqual(firenado_root, firenado.conf.ROOT)

    def test_firenado_config_file_default_value(self):
        """ Test if the default Firenado config file value will be "firenado".
        """
        self.assertEqual("firenado", firenado.conf.FIRENADO_CONFIG_FILE)

    def test_firenado_config_file_custom_value(self):
        """ Test if Firenado config file value will be changed setting
        FIRENADO_CONFIG_FILE env variable.
        """
        custom_file_name = "custom_file"
        os.environ['FIRENADO_CONFIG_FILE'] = custom_file_name
        reload(firenado.conf)
        self.assertEqual(firenado.conf.FIRENADO_CONFIG_FILE, custom_file_name)
        del os.environ['FIRENADO_CONFIG_FILE']
        reload(firenado.conf)

    def test_system_config_path_default_value(self):
        """ Test if the default system config path is /etc/firenado
        """
        self.assertEqual(firenado.conf.SYS_CONFIG_PATH, "/etc/firenado")

    def test_system_config_path_custom_value(self):
        """ Test if the default system config path will be changed setting the
        FIRENADO_SYS_CONFIG_PATH env variable
        """
        custom_sys_config_path = "/etc/anotherplace"
        os.environ['FIRENADO_SYS_CONFIG_PATH'] = custom_sys_config_path
        reload(firenado.conf)
        self.assertEqual(firenado.conf.SYS_CONFIG_PATH, custom_sys_config_path)
        del os.environ['FIRENADO_SYS_CONFIG_PATH']
        reload(firenado.conf)

    def test_only_framework_stack(self):
        """ Tests is only the framework config stack was loaded.
        No app config is provided.
        """
        self.assertEqual(firenado.conf.stack[0],
                          firenado.conf.LIB_CONFIG_FILE)

    def test_app_stack(self):
        """ Application config is provided. Test if the app config file was
        loaded.
        """
        chdir_app("yml", "conf")
        self.assertEqual(firenado.conf.stack[0],
                          firenado.conf.LIB_CONFIG_FILE)
        self.assertEqual(firenado.conf.stack[1],
                          firenado.conf.APP_CONFIG_FILE)

    def test_sys_stack(self):
        """ System config path is provided. Test if the system config file was
        loaded.
        """
        os.environ['FIRENADO_SYS_CONFIG_PATH'] = os.path.join(
            os.path.dirname(__file__), "resources", "conf", "sys_config")
        reload(firenado.conf)
        self.assertEqual(firenado.conf.stack[0],
                          firenado.conf.LIB_CONFIG_FILE)
        self.assertEqual(firenado.conf.stack[1],
                          firenado.conf.SYS_CONFIG_FILE)
        self.assertEqual("%(asctime)s - %(message)s",
                         firenado.conf.log['format'])
        self.assertEqual(logging.DEBUG, firenado.conf.log['level'])
        del os.environ['FIRENADO_SYS_CONFIG_PATH']
        reload(firenado.conf)

    def test_app_addresses_default(self):
        """ If no addresses are provided to the application we default to
        ipv4 and ipv6 loopbacks.
        """
        # There is no addresses configured into the conf/yml firenado.yml
        chdir_app("yml", "conf")
        self.assertTrue(firenado.conf.app['socket'] is None)
        self.assertEqual(len(firenado.conf.app['addresses']), 2)
        self.assertEqual(firenado.conf.app['addresses'][0], "::")
        self.assertEqual(firenado.conf.app['addresses'][1], "0.0.0.0")

    def test_app_addresses_from_conf(self):
        """ Getting localhost defined into the configuration.
        """
        # At the conf/root_url app.addresses has only localhost
        chdir_app("root_url", "conf")
        self.assertTrue(firenado.conf.app['socket'] is None)
        self.assertEqual(len(firenado.conf.app['addresses']), 1)
        self.assertEqual(firenado.conf.app['addresses'][0], "localhost")

    def test_app_port(self):
        """ Checks if the app port is set correctly.
        """
        # Loading file from test/resources/session/file/conf/firenado.yml
        chdir_app("file", "session")
        self.assertTrue(firenado.conf.app['socket'] is None)
        self.assertEqual(firenado.conf.app['port'], 8887)

    def test_app_pythonpath(self):
        """ Checks if the pythonpath is set on the application config file.
        """
        chdir_app("file", "session")
        self.assertEqual(firenado.conf.app['pythonpath'], "..")

    def test_yml_loaded(self):
        """ On an application with a yml and yaml config files the yml should
        be loaded.
        """
        chdir_app("yml", "conf")
        self.assertEqual("yml", fs.get_file_extension(
                firenado.conf.APP_CONFIG_FILE))

    def test_settings_empty(self):
        """ If no app settings is defined an empty dict is set.
        """
        chdir_app("yml", "conf")
        self.assertDictEqual({}, firenado.conf.app['settings'])

    def test_settings(self):
        """ If no app settings is defined an empty dict is set.
        """
        chdir_app("settings", "conf")
        settings_dict = {
            'cookie_secret': "cookie---secret",
            'debug': True,
            'xsrf_cookies': True
        }
        self.assertDictEqual(settings_dict, firenado.conf.app['settings'])

    def test_static_path(self):
        """ If static path is defined on the app configuration.
        """
        chdir_app("yml", "conf")
        self.assertEqual("yml_static_path", firenado.conf.app['static_path'])

    def test_root_url(self):
        """ Test if the root path was set on the app configuration.
        """
        chdir_app("root_url", "conf")
        self.assertEqual("a_root_url", firenado.conf.app['url_root_path'])

    def test_root_url_slash_in_front(self):
        """ Test if the root path with a slash in the front will be returned
        without it was set on the app configuration.
        """
        chdir_app("root_url_slash_in_front", "conf")
        self.assertEqual("a_root_url",  firenado.conf.app['url_root_path'])

    def test_root_url_slash_none(self):
        """ Test if the root path with a slash in the front will be returned
        without it was set on the app configuration.
        """
        chdir_app("root_url_slash_none", "conf")
        self.assertEqual(None,  firenado.conf.app['url_root_path'])

    def test_static_path(self):
        """ If static url prefix is defined on the app configuration.
        """
        chdir_app("yml", "conf")
        self.assertEqual("yml_static_url_prefix",
                          firenado.conf.app['static_url_prefix'])

    def test_session_type_file(self):
        """ Checks if the session is enabled and the type is file
        """
        chdir_app("file", "session")
        self.assertEqual(firenado.conf.session['enabled'], True)
        self.assertEqual(firenado.conf.session['type'], "file")

    def test_session_name_default(self):
        """ Checks if the session name is default, FIRENADOSESSID
        """
        chdir_app("file", "session")
        self.assertEqual(firenado.conf.session['enabled'], True)
        self.assertEqual(firenado.conf.session['name'], "FIRENADOSESSID")

    def test_session_type_redis(self):
        """ Checks if the session is enabled and the type is redis
        """
        chdir_app("redis", "session")
        self.assertEqual(firenado.conf.session['enabled'], True)
        self.assertEqual(firenado.conf.session['type'], "redis")

    def test_session_name_custom(self):
        """ Checks if the session name will be defined as in the config file
        """
        chdir_app("redis", "session")
        self.assertEqual(firenado.conf.session['enabled'], True)
        self.assertEqual(firenado.conf.session['name'], "REDISSESSID")


class MultiAppTestCase(unittest.TestCase):
    """ Case that tests multi app configuration.
    """

    def test_multi_app_true(self):
        """ Checks if the application is multi app
        """
        chdir_app("multiapp")
        self.assertTrue(firenado.conf.is_multi_app)

    def test_multi_app_false(self):
        """ Checks if the application isn't multi app
        """
        chdir_app("tornadoweb")
        self.assertFalse(firenado.conf.is_multi_app)



#!/usr/bin/env python
#
# Copyright 2015-2017 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def rooted_path(root, path):
    if root is None:
        root = "/"
    if root[0] != "/":
        root = "/%s" % root
    if root[-1] != "/":
        root = "%s/" % root
    rooted_path = "".join([root, path.lstrip("/")])
    if rooted_path == "/":
        return rooted_path
    return rooted_path.rstrip("/")
